﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;
using System.Reflection;
using System.Text.RegularExpressions;

namespace CodeReview.Binary
{
    /// <summary>
    /// Base class for the implementation of <see cref="ICheck"/>.
    /// </summary>
    public abstract class AbstractCheck : ICheck
    {
        /// <summary>
        /// The category of the check.
        /// </summary>
        private readonly string category;

        /// <summary>
        /// The short code associated with the check (2 letters, 4 digits).
        /// </summary>
        private readonly string shortCheckId;

        /// <summary>
        /// The full code associated with the check.
        /// </summary>
        private readonly string checkId;

        /// <summary>
        /// The action used to create a new <see cref="Message"/>
        /// if the check is not respected.
        /// </summary>
        private AddMessageDelegate addMessage;

        /// <summary>
        /// Initializes a new instance of the AbstractCheck class.
        /// </summary>
        /// <param name="category">
        /// The category of the check.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// One of the parameter is <c>null</c> or empty.
        /// </exception>
        /// <exception cref="NotSupportedException">
        /// The name of the check doesn't respect the naming convention.
        /// </exception>
        protected AbstractCheck(string category)
        {
            if (string.IsNullOrEmpty(category))
            {
                throw new ArgumentNullException("category");
            }

            this.category = category;

            Match match = Regex.Match(this.GetType().Name, "^([A-Z]{2}[0-9]{4})(.*)$");

            if (!match.Success)
            {
                string message = string.Format(Error.Culture, Error.CheckNameFormat, this.GetType().Name);
                throw new NotSupportedException(message);
            }

            this.shortCheckId = match.Groups[1].Value;
            string longCheckId = match.Groups[2].Value;
            this.checkId = string.Format(CultureInfo.InvariantCulture, "{0}:{1}", this.ShortCheckId, longCheckId);
        }

        /// <summary>
        /// Gets the category of the check.
        /// </summary>
        /// <seealso cref="ICheckDetail.Category"/>
        public string Category
        {
            get { return this.category; }
        }

        /// <summary>
        /// Gets the short code associated with the check (2 letters, 4 digits).
        /// </summary>
        /// <seealso cref="ICheckDetail.CheckId"/>
        public string ShortCheckId
        {
            get { return this.shortCheckId; }
        }

        /// <summary>
        /// Gets the full code associated with the check.
        /// </summary>
        /// <seealso cref="ICheckDetail.CheckId"/>
        public string CheckId
        {
            get { return this.checkId; }
        }

        /// <summary>
        /// Initializes the check for a specific parser.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        /// <param name="addMessage">
        /// The action used to create a new message if the check is not respected.
        /// </param>
        public void Initialize(IParser parser, AddMessageDelegate addMessage)
        {
            this.addMessage = addMessage;
            this.Initialize(parser);
        }

        /// <summary>
        /// Associates the check to a specific parser.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected abstract void Initialize(IParser parser);

        /// <summary>
        /// Registers a new message representing an issue found during
        /// the current check.
        /// </summary>
        /// <param name="messageCode">
        /// The code of the message.
        /// </param>
        /// <param name="target">
        /// The target of the message.
        /// </param>
        /// <param name="messageId">
        /// The identifier of the message.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// The <paramref name="target"/> parameter is <c>null</c>.
        /// </exception>
        protected void AddMessage(I18nMessage messageCode, MemberInfo target, string messageId)
        {
            if (target == null)
            {
                throw new ArgumentNullException("target");
            }

            string targetName = target.GetFullName();
            string localeMessage = I18nManager.Format(messageCode, targetName, messageId);
            this.addMessage(this, localeMessage, targetName, messageId);
        }

        /// <summary>
        /// Registers a new message representing an issue found during
        /// the current check.
        /// </summary>
        /// <param name="messageCode">
        /// The code of the message.
        /// </param>
        /// <param name="target">
        /// The target of the message.
        /// </param>
        protected void AddMessage(I18nMessage messageCode, MemberInfo target)
        {
            this.AddMessage(messageCode, target, null);
        }

        /// <summary>
        /// Registers a new message representing an issue found during
        /// the current check that targets the assembly.
        /// </summary>
        /// <param name="messageCode">
        /// The code of the message.
        /// </param>
        protected void AddMessageForAssembly(I18nMessage messageCode)
        {
            string localeMessage = I18nManager.GetString(messageCode);
            this.addMessage(this, localeMessage, null, null);
        }

        /// <summary>
        /// Registers a new message representing an issue found during
        /// the current check that targets a namespace.
        /// </summary>
        /// <param name="messageCode">
        /// The code of the message.
        /// </param>
        /// <param name="ns">
        /// The full name of the namespace.
        /// </param>
        protected void AddMessageForNamespace(I18nMessage messageCode, string ns)
        {
            string localeMessage = I18nManager.Format(messageCode, ns);
            this.addMessage(this, localeMessage, ns, null);
        }
    }
}
