﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Implements a code review check on abstract classes to
    /// avoid public constructors.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1012")]
    public class CR0001TypeAbstractShouldNotHavePublicConstructors : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current class is abstract.
        /// </summary>
        private bool isAbstract;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0001TypeAbstractShouldNotHavePublicConstructors class.
        /// </summary>
        public CR0001TypeAbstractShouldNotHavePublicConstructors()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on class and constructor parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseType;
            parser.ParseConstructorStarted += this.OnParseConstructor;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseType(Type type)
        {
            this.isAbstract = type.IsAbstract;
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructor(ConstructorInfo constructor)
        {
            if (this.isAbstract && constructor.IsPublic)
            {
                this.AddMessage(I18nMessage.CR0001, constructor);
            }
        }
    }
}
