﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the types that own a <see cref="IDisposable"/> property
    /// or field implement <see cref="IDisposable"/> too.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1001")]
    public class CR0002TypeWithDisposablesShouldBeDisposable : AbstractCheck
    {
        /// <summary>
        ///  A value indicating whether the current type contains
        ///  <see cref="IDisposable"/> members.
        /// </summary>
        private bool hasDisposableMembers;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0002TypeWithDisposablesShouldBeDisposable class.
        /// </summary>
        public CR0002TypeWithDisposablesShouldBeDisposable()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on class, field and property parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParsedField += this.OnParsedField;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.hasDisposableMembers = false;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (typeof(IDisposable).IsAssignableFrom(field.FieldType))
            {
                this.hasDisposableMembers = true;
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (typeof(IDisposable).IsAssignableFrom(property.PropertyType))
            {
                this.hasDisposableMembers = true;
            }
        }

        /// <summary>
        /// Called at the end of a type parsing.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (!typeof(IDisposable).IsAssignableFrom(type) && this.hasDisposableMembers)
            {
                this.AddMessage(I18nMessage.CR0002, type);
            }
        }
    }
}
