﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a class doesn't expose through field, property or method
    /// a generic list implementation.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1002")]
    public class CR0003MemberShouldNotExposeGenericListImplementation : AbstractCheck
    {
        /// <summary>
        /// The reference to the IEnumerable type.
        /// </summary>
        private static readonly Type reference = typeof(IEnumerable<>);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0003MemberShouldNotExposeGenericListImplementation class.
        /// </summary>
        public CR0003MemberShouldNotExposeGenericListImplementation()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on property and field parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedProperty += this.OnParseProperty;
            parser.ParsedField += this.OnParseField;
            parser.ParsedParameter += this.OnParsedParameter;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParseField(FieldInfo field)
        {
            // Test that the field is exposed
            if (field.IsPrivate)
            {
                return;
            }

            if (this.IsGenericCollectionClass(field.FieldType))
            {
                this.AddMessage(I18nMessage.CR0003Member, field);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParseProperty(PropertyInfo property)
        {
            bool exposed = false;

            // Test if the getter is exposed
            if (property.CanRead && property.GetGetMethod(true) != null)
            {
                exposed = !property.GetGetMethod(true).IsPrivate;
            }

            // Test if the setter is exposed
            if (property.CanWrite && property.GetSetMethod(true) != null)
            {
                exposed = exposed || !property.GetSetMethod(true).IsPrivate;
            }

            if (exposed && this.IsGenericCollectionClass(property.PropertyType))
            {
                this.AddMessage(I18nMessage.CR0003Member, property);
            }
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            MethodBase method = parameter.Member as MethodBase;
            if (!method.IsPrivate && this.IsGenericCollectionClass(parameter.ParameterType))
            {
                if (parameter.Name == null)
                {
                    // return of the method
                    this.AddMessage(I18nMessage.CR0003Return, method, "return");
                }
                else
                {
                    // normal parameter
                    this.AddMessage(I18nMessage.CR0003Parameter, method, parameter.Name);
                }
            }
        }

        /// <summary>
        /// Cheeks that a specific type, is not a generic collection class.
        /// </summary>
        /// <param name="type">
        /// The type to check.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="type"/> is a class part of the
        /// System.Collections.Generic namespace that inherits from
        /// IEnumerable&lt;&gt;.
        /// </returns>
        private bool IsGenericCollectionClass(Type type)
        {
            // Test that the type is a class part of the System.Collections.Generic namespace
            if (!type.IsClass || !type.Namespace.Equals("System.Collections.Generic"))
            {
                return false;
            }

            // Test if the class implements IEnumerable<T>
            foreach (Type inter in type.GetInterfaces())
            {
                if (inter.IsGenericType && reference == inter.GetGenericTypeDefinition())
                {
                    return true;
                }
            }

            return false;
        }
    }
}
