﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that generic types contains any static elements.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1000")]
    public class CR0004TypeGenericShouldNotContainStaticElement : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current type is a generic type.
        /// </summary>
        private bool isGeneric;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0004TypeGenericShouldNotContainStaticElement class.
        /// </summary>
        public CR0004TypeGenericShouldNotContainStaticElement()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type, field, property and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += new TypeHandler(this.OnParseTypeStarted);
            parser.ParsedField += new FieldHandler(this.OnParsedField);
            parser.ParsedProperty += new PropertyHandler(this.OnParsedProperty);
            parser.ParseMethodStarted += new MethodHandler(this.OnParseMethodStarted);
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isGeneric = type.IsGenericType;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (this.isGeneric && field.IsStatic)
            {
                this.AddMessage(I18nMessage.CR0004Field, field);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            bool isStatic = property.GetAccessors().FirstOrDefault(m => m.IsStatic) != null;
            if (this.isGeneric && isStatic)
            {
                this.AddMessage(I18nMessage.CR0004Property, property);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (this.isGeneric && method.IsStatic)
            {
                this.AddMessage(I18nMessage.CR0004Method, method);
            }
        }
    }
}
