﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that events are based on <see cref="System.EventHandler"/>
    /// or <see cref="System.EventHandler{T}"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1003")]
    public class CR0005EventShouldBeSystemEventHandler : AbstractCheck
    {
        /// <summary>
        /// The base type for simple event handler (<see cref="System.EventHandler"/>).
        /// </summary>
        private static readonly Type normalHandlerType
            = typeof(System.EventHandler);

        /// <summary>
        /// The base type for generic event handler (<see cref="System.EventHandler{T}"/>).
        /// </summary>
        private static readonly Type genericHandlerType
            = typeof(System.EventHandler<>);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0005EventShouldBeSystemEventHandler class.
        /// </summary>
        public CR0005EventShouldBeSystemEventHandler()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on event parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedEvent += this.OnParsedEvent;
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parserd event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            Type reference = eventInfo.EventHandlerType;
            if (reference == normalHandlerType)
            {
                return;
            }

            if (reference.IsGenericType && reference.GetGenericTypeDefinition() == genericHandlerType)
            {
                return;
            }

            this.AddMessage(I18nMessage.CR0005, eventInfo);
        }
    }
}
