﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a type doesn't expose a generic type based on a generic type.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1006")]
    public class CR0007MemberShouldNotExposeNestedGenericType : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0007MemberShouldNotExposeNestedGenericType class.
        /// </summary>
        public CR0007MemberShouldNotExposeNestedGenericType()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on constructor, method, field
        /// and parameter parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedField += this.OnParsedField;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParsedParameter += this.OnParsedParameter;
        }

        /// <summary>
        /// Checks whether a type is a generic with a generic argument.
        /// </summary>
        /// <param name="type">
        /// The checked type.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="type"/> is a generic type
        /// and if one of its generic parameters is also a generic type,
        /// otherwise <c>false</c>.
        /// </returns>
        private static bool IsNestedGenericType(Type type)
        {
            if (type.IsGenericType)
            {
                foreach (Type arg in type.GetGenericArguments())
                {
                    if (arg.IsGenericType)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (field.IsPrivate)
            {
                return;
            }

            if (IsNestedGenericType(field.FieldType))
            {
                this.AddMessage(I18nMessage.CR0007Field, field);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            bool exposed = false;

            // Test if the getter is exposed
            if (property.CanRead && property.GetGetMethod(true) != null)
            {
                exposed = !property.GetGetMethod(true).IsPrivate;
            }

            // Test if the setter is exposed
            if (property.CanWrite && property.GetSetMethod(true) != null)
            {
                exposed = exposed || !property.GetSetMethod(true).IsPrivate;
            }

            if (exposed && IsNestedGenericType(property.PropertyType))
            {
                this.AddMessage(I18nMessage.CR0007Property, property);
            }
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            MethodBase method = parameter.Member as MethodBase;
            if (method == null || method.IsPrivate)
            {
                return;
            }

            if (IsNestedGenericType(parameter.ParameterType))
            {
                if (string.IsNullOrEmpty(parameter.Name))
                {
                    this.AddMessage(I18nMessage.CR0007Return, parameter.Member);
                }
                else
                {
                    this.AddMessage(I18nMessage.CR0007Parameter, parameter.Member, parameter.Name);
                }
            }
        }
    }
}
