﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the reference parameters are based on generic parameters.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1007")]
    public class CR0008ParameterWithRefShouldUseGenerics : AbstractCheck
    {
        /// <summary>
        /// The generic parameters of the parsed method.
        /// </summary>
        private ICollection<Type> genericParameters;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0008ParameterWithRefShouldUseGenerics class.
        /// </summary>
        public CR0008ParameterWithRefShouldUseGenerics()
            : base(Constants.Design)
        {
            this.genericParameters = new LinkedList<Type>();
        }

        /// <summary>
        /// Initializes the check to react on method and parameter parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
            parser.ParsedParameter += this.OnParsedParameter;
            parser.ParseMethodEnded += this.OnParseMethodEnded;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            this.genericParameters.Clear();
            foreach (Type type in method.GetGenericArguments())
            {
                this.genericParameters.Add(type);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodEnded(MethodInfo method)
        {
            this.genericParameters.Clear();
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            // Avoid to check a return type.
            if (string.IsNullOrEmpty(parameter.Name))
            {
                return;
            }

            if (parameter.ParameterType.IsByRef)
            {
                Type parameterType = parameter.ParameterType.GetElementType();
                if (!this.genericParameters.Contains(parameterType))
                {
                    this.AddMessage(I18nMessage.CR0008, parameter.Member, parameter.Name);
                }
            }
        }
    }
}
