﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that the generic parameters of a method are associated to parameters.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1004")]
    public class CR0009MethodGenericShouldUseInference : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current parsed method is generic.
        /// </summary>
        private bool isGeneric;

        /// <summary>
        /// The generic parameters of the parsed method.
        /// </summary>
        private ICollection<Type> genericParameters;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0009MethodGenericShouldUseInference class.
        /// </summary>
        public CR0009MethodGenericShouldUseInference()
            : base(Constants.Design)
        {
            this.genericParameters = new LinkedList<Type>();
        }

        /// <summary>
        /// Initializes the check to react on method and parameter parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
            parser.ParsedParameter += this.OnParsedParameter;
            parser.ParseMethodEnded += this.OnParseMethodEnded;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!method.IsGenericMethodDefinition)
            {
                return;
            }

            this.isGeneric = true;
            this.genericParameters.Clear();
            foreach (Type type in method.GetGenericArguments())
            {
                this.genericParameters.Add(type);
            }
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            if (this.genericParameters.Contains(parameter.ParameterType))
            {
                this.genericParameters.Remove(parameter.ParameterType);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodEnded(MethodInfo method)
        {
            if (this.isGeneric)
            {
                foreach (Type type in this.genericParameters)
                {
                    this.AddMessage(I18nMessage.CR0009, method, type.Name);
                }
            }

            this.isGeneric = false;
            this.genericParameters.Clear();
        }
    }
}
