﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that an enumeration defines one and only one name with the zero value.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1008")]
    public class CR0010EnumShouldHaveZeroValue : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the parsed type is an enumeration.
        /// </summary>
        private bool isEnum;

        /// <summary>
        /// The field associated with zero as been parsed for the current type.
        /// </summary>
        private bool isZeroFound;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0010EnumShouldHaveZeroValue class.
        /// </summary>
        public CR0010EnumShouldHaveZeroValue()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type and field parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParsedField += this.OnParsedField;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isZeroFound = false;
            this.isEnum = type.IsEnum;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (this.isEnum && field.IsStatic)
            {
                int value = (int)field.GetValue(null);
                if (value == 0)
                {
                    if (this.isZeroFound)
                    {
                        this.AddMessage(I18nMessage.CR0010Double, field.DeclaringType, field.Name);
                    }
                    else
                    {
                        this.isZeroFound = true;
                    }
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isEnum && !this.isZeroFound)
            {
                this.AddMessage(I18nMessage.CR0010Missing, type);
            }
        }
    }
}
