﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that the flags enumeration have a <c>None</c> field and that this
    /// field is associated to the zero value.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1008")]
    public class CR0011EnumFlagShouldHaveANoneValue : AbstractCheck
    {
        /// <summary>
        /// The expected name for the field associated with the zero value in a flag enumeration.
        /// </summary>
        public const string FlagZeroName = "None";

        /// <summary>
        /// The constructor of the <see cref="FlagsAttribute"/> class.
        /// </summary>
        private static readonly ConstructorInfo Reference
            = typeof(FlagsAttribute).GetConstructor(Type.EmptyTypes);

        /// <summary>
        /// A value indicating whether the current parsed type if a flag.
        /// </summary>
        private bool isFlag;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0011EnumFlagShouldHaveANoneValue class.
        /// </summary>
        public CR0011EnumFlagShouldHaveANoneValue()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type and field parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParsedField += this.OnParsedField;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isFlag = type.IsEnum && type.GetCustomAttributesData().Count(c => c.Constructor == Reference) > 0;

            // The field None is not defined
            if (this.isFlag && !type.IsEnumDefined(FlagZeroName))
            {
                this.AddMessage(I18nMessage.CR0011Missing, type);
            }
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (this.isFlag && field.IsStatic)
            {
                int value = (int)field.GetValue(null);
                bool isZeroName = string.Equals(field.Name, FlagZeroName, StringComparison.Ordinal);
                if (isZeroName && value != 0)
                {
                    this.AddMessage(I18nMessage.CR0011Value, field.DeclaringType, field.Name);
                }
            }
        }
    }
}
