﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the events are associated to event handler that have a proper
    /// signature.
    /// </summary>
    /// <remarks>
    /// A proper signature is:
    /// <list type="bullet">
    /// <item>return type is <see cref="System.Void"/></item>
    /// <item>The handler have two arguments</item>
    /// <item>The first argument is called 'sender' and is an object</item>
    /// <item>The second argument is called 'e' and inherit of <see cref="EventArgs"/></item>
    /// </list>
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1009")]
    public class CR0012EventHandlerShouldHaveAProperSignature : AbstractCheck
    {
        /// <summary>
        /// The cache of the parsed delegates.
        /// </summary>
        private IDictionary<Type, CheckFlags> cache;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0012EventHandlerShouldHaveAProperSignature class.
        /// </summary>
        public CR0012EventHandlerShouldHaveAProperSignature()
            : base(Constants.Design)
        {
            this.cache = new Dictionary<Type, CheckFlags>();
        }

        /// <summary>
        /// The status associated with the parsed delegates.
        /// </summary>
        [Flags]
        private enum CheckFlags
        {
            /// <summary>
            /// The default value.
            /// </summary>
            None = 0,

            /// <summary>
            /// The delegate is declared in the parsed assembly.
            /// </summary>
            IsDeclared,

            /// <summary>
            /// The delegate is used as an event handler in a type declared
            /// in the parsed assembly.
            /// </summary>
            IsHandler,

            /// <summary>
            /// The delegate is an event declared and used in the parsed
            /// assembly.
            /// </summary>
            IsDeclaredHandler = IsDeclared | IsHandler,
        }

        /// <summary>
        /// Initializes the check to react on type and event parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParsedEvent += this.OnParsedEvent;
        }

        /// <summary>
        /// Checks that a specific event handler have the required signature.
        /// </summary>
        /// <param name="type">
        /// The checked delegate type.
        /// </param>
        private void Check(Type type)
        {
            MethodInfo method = type.GetMethod("Invoke");
            ParameterInfo[] parameters = method.GetParameters();
            if (method.ReturnType != typeof(void)
                || parameters.Length != 2
                || parameters[0].ParameterType != typeof(object)
                || !string.Equals(parameters[0].Name, "sender", StringComparison.OrdinalIgnoreCase)
                || !typeof(EventArgs).IsAssignableFrom(parameters[1].ParameterType)
                || !string.Equals(parameters[1].Name, "e", StringComparison.OrdinalIgnoreCase))
            {
                this.AddMessage(I18nMessage.CR0012, type);
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (!this.cache.ContainsKey(type))
            {
                // first encounter
                this.cache.Add(type, CheckFlags.IsDeclared);
            }
            else if (this.cache[type] == CheckFlags.IsHandler)
            {
                // already encounter as an event handler
                // should be checked.
                this.Check(type);
                this.cache[type] = CheckFlags.IsDeclaredHandler;
            }
            else
            {
                // the type could be flag as:
                // - IsDeclared - it has already be parsed by this method - nothing to do
                // - IsDeclaredHandler - it has already be checked - nothing to do
            }
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            Type type = eventInfo.EventHandlerType;
            if (type.IsGenericType)
            {
                type = type.GetGenericTypeDefinition();
            }

            if (!this.cache.ContainsKey(type))
            {
                // first encounter
                this.cache.Add(type, CheckFlags.IsHandler);
            }
            else if (this.cache[type] == CheckFlags.IsDeclared)
            {
                // already encounter as a declared delegate
                // should be checked.
                this.Check(type);
                this.cache[type] = CheckFlags.IsDeclaredHandler;
            }
            else
            {
                // the type could be flag as:
                // - IsHandler - it has already be parsed by this method - nothing to do
                // - IsDeclaredHandler - it has already be checked - nothing to do
            }
        }
    }
}
