﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Collections.Generic;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a class implementing <see cref="IEnumerable"/> implements
    /// also <see cref="IEnumerable{T}"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1010")]
    public class CR0013TypeEnumerableShouldBeGeneric : AbstractCheck
    {
        /// <summary>
        /// The non generic reference.
        /// </summary>
        private static readonly Type reference = typeof(IEnumerable);

        /// <summary>
        /// The generic reference.
        /// </summary>
        private static readonly Type genericReference = typeof(IEnumerable<>);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0013TypeEnumerableShouldBeGeneric class.
        /// </summary>
        public CR0013TypeEnumerableShouldBeGeneric()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (reference.IsAssignableFrom(type))
            {
                bool implementsGeneric = false;
                foreach (Type @interface in type.GetInterfaces())
                {
                    // Checks if the class implements IEnumerable<> for a specific parameter.
                    if (@interface.IsGenericType && @interface.GetGenericTypeDefinition() == genericReference)
                    {
                        implementsGeneric = true;
                    }

                    // Checks if the class implements IEnumerable<> with a generic parameter.
                    if (@interface.IsGenericTypeDefinition && @interface == genericReference)
                    {
                        implementsGeneric = true;
                    }
                }

                if (!implementsGeneric)
                {
                    this.AddMessage(I18nMessage.CR0013, type);
                }
            }
        }
    }
}
