﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a type implementing the add or substract operator, provides
    /// the equality operator as well.
    /// </summary>
    /// <remarks>
    /// The compilator ensures that:
    /// <list type="bullet">
    /// <item>The operator != is also defined (CS0216 - error)</item>
    /// <item>The method Equals is overriden if the operator != or == are defined (CS0660 - warning)</item>
    /// <item>The method GetHashCode is overriden if the operator != or == are defined (CS0661 - warning)</item>
    /// </list>
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1013")]
    public class CR0014TypeWithAddOrSubstractShouldImplementEquality : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current type contains
        /// an equality operator.
        /// </summary>
        private bool containsEqualityOperator;

        /// <summary>
        /// A value indicating whether the current type contains
        /// at least one of the add or substract operators.
        /// </summary>
        private bool containsAddOrSubstractOperator;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0014TypeWithAddOrSubstractShouldImplementEquality class.
        /// </summary>
        public CR0014TypeWithAddOrSubstractShouldImplementEquality()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.containsAddOrSubstractOperator = false;
            this.containsEqualityOperator = false;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.containsAddOrSubstractOperator && !this.containsEqualityOperator)
            {
                this.AddMessage(I18nMessage.CR0014, type);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (method.IsStatic && method.IsSpecialName)
            {
                if (method.Name == "op_Addition" || method.Name == "op_Subtraction")
                {
                    this.containsAddOrSubstractOperator = true;
                }
                else if (method.Name == "op_Equality")
                {
                    this.containsEqualityOperator = true;
                }
            }
        }
    }
}
