﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that the assembly is tagged with a <see cref="CLSCompliantAttribute"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1014")]
    public class CR0015AssemblyShouldHaveCLSCompliantAttribute : AbstractCheck
    {
        /// <summary>
        /// The reference to the constructor the attribute.
        /// </summary>
        private static readonly ConstructorInfo reference
            = typeof(CLSCompliantAttribute).GetConstructor(new Type[] { typeof(bool) });

        /// <summary>
        /// A value indicating whether the assembly contains the expected
        /// attribute.
        /// </summary>
        private bool containsAttribute;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0015AssemblyShouldHaveCLSCompliantAttribute class.
        /// </summary>
        public CR0015AssemblyShouldHaveCLSCompliantAttribute()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on attribute parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedAssemblyAttribute += this.OnParsedAssemblyAttribute;
            parser.ParseAssemblyStarted += this.OnParseAssemblyStarted;
            parser.ParseAssemblyEnded += this.OnParseAssemblyEnded;
        }

        /// <summary>
        /// Called when an assembly attribute is parsed.
        /// </summary>
        /// <param name="attribute">
        /// The parsed attribute.
        /// </param>
        private void OnParsedAssemblyAttribute(CustomAttributeData attribute)
        {
            if (attribute.Constructor == reference)
            {
                this.containsAttribute = true;
            }
        }

        /// <summary>
        /// Called after an assembly is parsed.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyStarted(Assembly assembly)
        {
            this.containsAttribute = false;
        }

        /// <summary>
        /// Called after an assembly is parsed.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyEnded(Assembly assembly)
        {
            if (!this.containsAttribute)
            {
                this.AddMessageForAssembly(I18nMessage.CR0015);
            }
        }
    }
}
