﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that the assembly is tagged with a <see cref="AssemblyVersionAttribute"/>.
    /// </summary>
    /// <remarks>
    /// The <c>AssemblyVersionAttribute</c> is not available through reflection, the test
    /// is based on the version of the assembly itself as a missing <c>AssemblyVersionAttribute</c>
    /// isequivalent to a 0.0.0.0 version number.
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1016")]
    public class CR0016AssemblyShouldHaveAssemblyVersionAttribute : AbstractCheck
    {
        /// <summary>
        /// The reference version number.
        /// </summary>
        private static Version reference = Version.Parse("0.0.0.0");

        /// <summary>
        /// Initializes a new instance of the
        /// CR0016AssemblyShouldHaveAssemblyVersionAttribute class.
        /// </summary>
        public CR0016AssemblyShouldHaveAssemblyVersionAttribute()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on attribute parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseAssemblyStarted += this.OnParseAssemblyStarted;
        }

        /// <summary>
        /// Called after an assembly is parsed.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyStarted(Assembly assembly)
        {
            if (assembly.GetName().Version == reference)
            {
                this.AddMessageForAssembly(I18nMessage.CR0016);
            }
        }
    }
}
