﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that an attribute has a <see cref="AttributeUsageAttribute"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1018")]
    public class CR0018AttributeShouldHaveAttributeUsageAttribute : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="Attribute"/> class.
        /// </summary>
        private static Type attributeType = typeof(Attribute);

        /// <summary>
        /// The reference to the <see cref="AttributeUsageAttribute"/> class.
        /// </summary>
        private static Type reference = typeof(AttributeUsageAttribute);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0018AttributeShouldHaveAttributeUsageAttribute class.
        /// </summary>
        public CR0018AttributeShouldHaveAttributeUsageAttribute()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (type.IsClass && !type.IsAbstract && attributeType.IsAssignableFrom(type))
            {
                CustomAttributeData data = type.GetCustomAttributesData().FirstOrDefault(a => a.Constructor.DeclaringType == reference);
                if (data == null)
                {
                    this.AddMessage(I18nMessage.CR0018, type);
                }
            }
        }
    }
}
