﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that an attribute provides only getter for positional elements.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1019")]
    public class CR0019AttributeShouldHaveReadOnlyAccessors : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current type is an Attribute.
        /// </summary>
        private bool isAttribute;

        /// <summary>
        /// All the arguments of the constructors stored as property names.
        /// </summary>
        private ICollection<string> positionalElements;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0019AttributeShouldHaveReadOnlyAccessors class.
        /// </summary>
        public CR0019AttributeShouldHaveReadOnlyAccessors()
            : base(Constants.Design)
        {
            this.positionalElements = new LinkedList<string>();
        }

        /// <summary>
        /// Initializes the parser to react on type, constructor and property parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseConstructorStarted += this.OnParseConstructorStarted;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isAttribute = typeof(Attribute).IsAssignableFrom(type);
            this.positionalElements.Clear();
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        /// <remarks>
        /// This method doesn't use Parameter parsing to avoid the parsing of methods.
        /// </remarks>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            if (!this.isAttribute)
            {
                return;
            }

            foreach (ParameterInfo parameter in constructor.GetParameters())
            {
                StringBuilder name = new StringBuilder(parameter.Name);
                name[0] = char.ToUpperInvariant(name[0]);

                this.positionalElements.Add(name.ToString());
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!this.isAttribute)
            {
                return;
            }

            string name = property.Name;
            if (this.positionalElements.Contains(name))
            {
                this.positionalElements.Remove(name);

                // Property associated to a positional element
                if (!property.CanRead || !property.GetGetMethod().IsPublic || property.CanWrite)
                {
                    this.AddMessage(I18nMessage.CR0019Property, property);
                }
            }
            else
            {
                // Property associated to a named parameter
                // see CR0020
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            foreach (string name in this.positionalElements)
            {
                this.AddMessage(I18nMessage.CR0019Argument, type, name);
            }
        }
    }
}
