﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a namespace contains at least five types.
    /// </summary>
    /// <remarks>
    /// This check doesn't take into account the default namespace and
    /// the nested types.
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1020")]
    public class CR0021NamespaceShouldHaveMoreThanFourTypes : AbstractCheck
    {
        /// <summary>
        /// The number of types per namespace.
        /// </summary>
        private IDictionary<string, int> numberOfTypes;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0021NamespaceShouldHaveMoreThanFourTypes class.
        /// </summary>
        public CR0021NamespaceShouldHaveMoreThanFourTypes()
            : base(Constants.Design)
        {
            this.numberOfTypes = new Dictionary<string, int>();
        }

        /// <summary>
        /// Initializes the parser to react on type and assembly parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseAssemblyStarted += this.OnParseAssemblyStarted;
            parser.ParseAssemblyEnded += this.OnParseAssemblyEnded;
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called before an assembly is parsed.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyStarted(Assembly assembly)
        {
            this.numberOfTypes.Clear();
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (type.IsNested)
            {
                return;
            }

            string ns = type.Namespace ?? string.Empty;

            if (this.numberOfTypes.ContainsKey(ns))
            {
                this.numberOfTypes[ns]++;
            }
            else
            {
                this.numberOfTypes.Add(ns, 1);
            }
        }

        /// <summary>
        /// Called after an assembly is parsed.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyEnded(Assembly assembly)
        {
            // Find the parent namespaces
            IList<string> parents = new List<string>();
            foreach (KeyValuePair<string, int> pair in this.numberOfTypes)
            {
                bool found = false;
                for (int i = 0; i < parents.Count; i++)
                {
                    if (pair.Key.StartsWith(parents[i]))
                    {
                        found = true;
                    }
                    else if (parents[i].StartsWith(pair.Key))
                    {
                        parents[i] = pair.Key;
                    }
                }

                if (!found)
                {
                    parents.Add(pair.Key);
                }
            }

            foreach (KeyValuePair<string, int> pair in this.numberOfTypes)
            {
                if (pair.Value < 5 && !parents.Contains(pair.Key))
                {
                    this.AddMessageForNamespace(I18nMessage.CR0021, pair.Key);
                }
            }
        }
    }
}
