﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a method doesn't have its three (or more) last parameters of the same type.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1025")]
    public class CR0026MethodShouldAvoidRepetitiveParameters : AbstractCheck
    {
        /// <summary>
        /// The current parsed type is a delegate.
        /// </summary>
        private bool isDelegate;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0026MethodShouldAvoidRepetitiveParameters class.
        /// </summary>
        public CR0026MethodShouldAvoidRepetitiveParameters()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isDelegate = typeof(Delegate).IsAssignableFrom(type);
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            // For a delegate, only the Invoke method is checked.
            if (this.isDelegate && !string.Equals(method.Name, "Invoke", StringComparison.Ordinal))
            {
                return;
            }

            ParameterInfo[] parameters = method.GetParameters();
            if (parameters.Length < 4)
            {
                return;
            }

            Type reference = parameters[parameters.Length - 1].ParameterType;
            for (int i = parameters.Length - 2; i > parameters.Length - 4; i--)
            {
                if (parameters[i].ParameterType != reference)
                {
                    return;
                }
            }

            if (this.isDelegate)
            {
                // For a delegate, add the message on the delegate not on the Invoke method.
                this.AddMessage(I18nMessage.CR0026, method.DeclaringType);
            }
            else
            {
                this.AddMessage(I18nMessage.CR0026, method);
            }
        }
    }
}
