﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that an enumeration with values that are power of two or combinaison of existing values
    /// are flagged with the <see cref="FlagsAttribute"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1027")]
    public class CR0027EnumShouldUseFlagsAttribute : AbstractCheck
    {
        /// <summary>
        /// The default constructor of the <see cref="FlagsAttribute"/> class.
        /// </summary>
        private static readonly ConstructorInfo reference = typeof(FlagsAttribute).GetConstructor(Type.EmptyTypes);

        /// <summary>
        /// A value indicating whether the parsed type is an enumeration without the Flags attribute.
        /// </summary>
        private bool isEnum;

        /// <summary>
        /// The list of all the parsed values that are a power of two.
        /// </summary>
        private ICollection<ulong> powerTwoValues;

        /// <summary>
        /// The list of all the other values.
        /// </summary>
        private ICollection<ulong> otherValues;

        /// <summary>
        /// Initializes a new instance of the CR0027EnumShouldUseFlagsAttribute class.
        /// </summary>
        public CR0027EnumShouldUseFlagsAttribute()
            : base(Constants.Design)
        {
            this.powerTwoValues = new LinkedList<ulong>();
            this.otherValues = new LinkedList<ulong>();
        }

        /// <summary>
        /// Initializes the check to react on type and field parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParsedField += this.OnParsedField;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.powerTwoValues.Clear();
            this.otherValues.Clear();

            if (type.IsEnum)
            {
                this.isEnum = type.GetCustomAttributesData().FirstOrDefault(d => d.Constructor == reference) == null;
            }
            else
            {
                this.isEnum = false;
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (!this.isEnum)
            {
                return;
            }

            // Test if the values are continuous
            bool isContinuous = true;
            bool start = true;
            ulong last = 0;
            foreach (ulong value in this.powerTwoValues.Union(this.otherValues).OrderBy(v => v))
            {
                if (start)
                {
                    start = false;
                }
                else if (value != last + 1)
                {
                    isContinuous = false;
                    break;
                }

                last = value;
            }

            if (isContinuous)
            {
                return;
            }

            // Test if the other values are composed by the declared power of two values
            bool isCandidate = true;
            foreach (ulong value in this.otherValues)
            {
                if (value != 0)
                {
                    ulong temp = value;
                    foreach (ulong power in this.powerTwoValues.OrderByDescending(v => v))
                    {
                        if (temp >= power)
                        {
                            temp = temp - power;
                        }
                    }

                    if (temp != 0)
                    {
                        isCandidate = false;
                        break;
                    }
                }
            }

            if (isCandidate)
            {
                this.AddMessage(I18nMessage.CR0027, type);
            }
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (!this.isEnum || !field.IsStatic)
            {
                return;
            }

            ulong value = (field.GetRawConstantValue() as IConvertible).ToUInt64(CultureInfo.InvariantCulture);

            // if value is a power of two is binary representation is 0--010--0.
            // in such a case (value - 0--01 & value) == 0--0
            if (value != 0x00UL && ((value - 0x01UL) & value) == 0x00UL)
            {
                this.powerTwoValues.Add(value);
            }
            else
            {
                this.otherValues.Add(value);
            }
        }
    }
}
