﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Proposes to change methods starting with <c>AddOn</c>, <c>RemoveOn</c>, <c>Fire</c>
    /// and <c>Raise</c> by events.
    /// </summary>
    /// <remarks>
    /// This checks consider a method <c>RaiseXxx</c> or <c>FireXxx</c> valid is an event
    /// <c>Xxx</c> exists in the same class.
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1030")]
    public class CR0030MethodShouldBeReplacedByEvent : AbstractCheck
    {
        /// <summary>
        /// The prefixes that denotes a method that can be replaced by an event.
        /// </summary>
        private static readonly string[] eventPrefixes = new string[]
        {
            "AddOn",
            "RemoveOn",
            "Fire",
            "Raise",
        };

        /// <summary>
        /// The prefixes that denotes a method that could be valid, if the associated
        /// event exists.
        /// </summary>
        private static readonly string[] validPrefixes = new string[]
        {
            "Fire",
            "Raise",
        };

        /// <summary>
        /// Initializes a new instance of the
        /// CR0030MethodShouldBeReplacedByEvent class.
        /// </summary>
        public CR0030MethodShouldBeReplacedByEvent()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            // Test if the method starts with the lookup values
            string prefix = eventPrefixes.FirstOrDefault(s => method.Name.StartsWith(s));
            if (prefix != null)
            {
                // Test if the method can be valid if a specific event exists.
                if (validPrefixes.Contains(prefix))
                {
                    string eventName = method.Name.Substring(prefix.Length);
                    EventInfo eventInfo = method.DeclaringType.GetEvent(eventName, Parser.AllDeclaredBindingFlags);

                    if (eventInfo != null)
                    {
                        // The method is valid
                        return;
                    }
                }

                this.AddMessage(I18nMessage.CR0030, method);
            }
        }
    }
}
