﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;
using System.Runtime.Serialization;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that Exceptions implements the four standard constructors.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1032")]
    public class CR0031TypeExceptionShouldHaveStandardConstructors : AbstractCheck
    {
        /// <summary>
        /// The signatures of the expected constructors.
        /// </summary>
        private static readonly Signature[] signatures;

        /// <summary>
        /// A value indicating whether the current parsed type is an exception.
        /// </summary>
        private bool isException;

        /// <summary>
        /// The number of good signatures found in the current parsed type.
        /// </summary>
        private int numberOfFoundSignatures;

        /// <summary>
        /// Initializes static members of the
        /// CR0031TypeExceptionShouldHaveStandardConstructors class.
        /// </summary>
        static CR0031TypeExceptionShouldHaveStandardConstructors()
        {
            signatures = new Signature[4];

            signatures[0] = new Signature() { IsSerializer = false, Parameters = new Parameter[0] };

            signatures[1] = new Signature() { IsSerializer = false, Parameters = new Parameter[1] };
            signatures[1].Parameters[0] = new Parameter() { Name = "message", Type = typeof(string) };

            signatures[2] = new Signature() { IsSerializer = false, Parameters = new Parameter[2] };
            signatures[2].Parameters[0] = new Parameter() { Name = "message", Type = typeof(string) };
            signatures[2].Parameters[1] = new Parameter() { Name = "innerException", Type = typeof(Exception) };

            signatures[3] = new Signature() { IsSerializer = true, Parameters = new Parameter[2] };
            signatures[3].Parameters[0] = new Parameter() { Name = "info", Type = typeof(SerializationInfo) };
            signatures[3].Parameters[1] = new Parameter() { Name = "context", Type = typeof(StreamingContext) };
        }

        /// <summary>
        /// Initializes a new instance of the
        /// CR0031TypeExceptionShouldHaveStandardConstructors class.
        /// </summary>
        public CR0031TypeExceptionShouldHaveStandardConstructors()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and constructor parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParseConstructorStarted += this.OnParseConstructorStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.numberOfFoundSignatures = 0;
            this.isException = typeof(Exception).IsAssignableFrom(type);
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            if (!this.isException)
            {
                return;
            }

            ParameterInfo[] parameters = constructor.GetParameters();

            foreach (Signature signature in signatures)
            {
                // Check the visibility of the constructor // current signature
                if (!signature.IsSerializer && !constructor.IsPublic)
                {
                    continue;
                }
                else if (signature.IsSerializer && constructor.DeclaringType.IsSealed && !constructor.IsPrivate)
                {
                    continue;
                }
                else if (signature.IsSerializer && !constructor.DeclaringType.IsSealed && !constructor.IsFamily)
                {
                    continue;
                }

                // Check the parameters of the constructor
                if (parameters.Length != signature.Parameters.Length)
                {
                    continue;
                }

                bool parametersMatch = true;
                for (int i = 0; i < parameters.Length; i++)
                {
                    if (parameters[i].Name != signature.Parameters[i].Name
                        || parameters[i].ParameterType != signature.Parameters[i].Type)
                    {
                        parametersMatch = false;
                    }
                }

                if (parametersMatch)
                {
                    this.numberOfFoundSignatures++;
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isException && this.numberOfFoundSignatures != signatures.Length)
            {
                this.AddMessage(I18nMessage.CR0031, type);
            }
        }

        /// <summary>
        /// Represents a constructor signature.
        /// </summary>
        private class Signature
        {
            /// <summary>
            /// Gets or sets a value indicating whether the signature is
            /// associated to the serializer constructor.
            /// </summary>
            public bool IsSerializer { get; set; }

            /// <summary>
            /// Gets or sets the parameters of the signature.
            /// </summary>
            public Parameter[] Parameters { get; set; }
        }

        /// <summary>
        /// Represents a parameter part of a constructor signature.
        /// </summary>
        private class Parameter
        {
            /// <summary>
            /// Gets or sets the type of the parameter.
            /// </summary>
            public Type Type { get; set; }

            /// <summary>
            /// Gets or sets the name of the parameter.
            /// </summary>
            public string Name { get; set; }
        }
    }
}
