﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that explicit interface implementations defined in non-sealed
    /// exposed classes have a public or protected method overridable with
    /// the same signature.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1033")]
    public class CR0032MethodExplicitShouldBeOverridable : AbstractCheck
    {
        /// <summary>
        /// A value indicating that the current type is valid for the check.
        /// </summary>
        private bool isValidType;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0032MethodExplicitShouldBeOverridable class.
        /// </summary>
        public CR0032MethodExplicitShouldBeOverridable()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (type.IsClass && (type.IsPublic || type.IsNestedPublic) && !type.IsSealed)
            {
                Type[] interfaces = type.GetInterfaces();
                this.isValidType = interfaces != null && interfaces.Length > 0;
            }
            else
            {
                this.isValidType = false;
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!this.isValidType)
            {
                return;
            }

            // Test if the method is an explicit implementation
            int pos = method.Name.LastIndexOf('.');
            if (pos == -1)
            {
                return;
            }

            // Extract the name of the method and try to find
            // a protected method with the same name.
            string name = method.Name.Substring(pos + 1);
            Type[] types = method.GetParameters().Select(p => p.ParameterType).ToArray();
            BindingFlags flags = BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly;
            MethodInfo parent = method.DeclaringType.GetMethod(name, flags, null, types, null);

            if (parent == null || parent.IsPrivate || parent.IsAssembly || !parent.IsVirtual)
            {
                this.AddMessage(I18nMessage.CR0032, method);
            }
        }
    }
}
