﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that implementation of <see cref="ICollection"/> implements
    /// strongly typed version of the <c>CopyTo</c> method.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1035")]
    public class CR0034TypeICollectionShouldBeStronglyTyped : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="ICollection"/> type.
        /// </summary>
        private static readonly Type reference = typeof(ICollection);

        /// <summary>
        /// A value indicating whether the parsed type implements
        /// <see cref="ICollection"/>.
        /// </summary>
        private bool isCollection;

        /// <summary>
        /// A value indicating whether the parsed type implements
        /// the strongly typed version of the <c>CopyTo</c> method.
        /// </summary>
        private bool implementCopyTo;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0034TypeICollectionShouldBeStronglyTyped class.
        /// </summary>
        public CR0034TypeICollectionShouldBeStronglyTyped()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isCollection = type.GetInterfaces().Where(i => i == reference).Count() > 0;
            this.implementCopyTo = false;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!this.isCollection)
            {
                return;
            }

            if (string.Equals(method.Name, "CopyTo", StringComparison.Ordinal))
            {
                ParameterInfo[] parameters = method.GetParameters();
                if (parameters != null && parameters.Length == 2)
                {
                    Type firstType = parameters[0].ParameterType;
                    bool firstParameterOk
                        = string.Equals(parameters[0].Name, "array", StringComparison.Ordinal)
                        && firstType.IsArray
                        && firstType.GetElementType() != typeof(object);

                    Type secondType = parameters[1].ParameterType;
                    bool secondParameterOk
                        = string.Equals(parameters[1].Name, "index", StringComparison.Ordinal)
                        && secondType == typeof(int);

                    if (firstParameterOk && secondParameterOk)
                    {
                        this.implementCopyTo = true;
                    }
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isCollection && !this.implementCopyTo)
            {
                this.AddMessage(I18nMessage.CR0034, type);
            }
        }
    }
}
