﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a class implementing <see cref="IEnumerator"/> provides
    /// a strongly typed version of the <c>Current</c> property.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1038")]
    public class CR0035TypeIEnumeratorShouldBeStronglyTyped : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="IEnumerator"/> type.
        /// </summary>
        private static readonly Type reference = typeof(IEnumerator);

        /// <summary>
        /// A value indicating whether the current parsed type implements
        /// <see cref="IEnumerator"/>.
        /// </summary>
        private bool isEnumerator;

        /// <summary>
        /// A value indicating whether the current parsed type implements
        /// a strongly typed version of the <c>Current</c> property.
        /// </summary>
        private bool implementCurrent;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0035TypeIEnumeratorShouldBeStronglyTyped class.
        /// </summary>
        public CR0035TypeIEnumeratorShouldBeStronglyTyped()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and property parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParsedProperty += this.OnParsedProperty;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isEnumerator = type.GetInterfaces().Where(i => i == reference).Count() > 0;
            this.implementCurrent = false;
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!this.isEnumerator)
            {
                return;
            }

            if (string.Equals(property.Name, "Current", StringComparison.Ordinal))
            {
                if (property.CanRead && property.GetGetMethod().IsPublic && property.PropertyType != typeof(object))
                {
                    this.implementCurrent = true;
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isEnumerator && !this.implementCurrent)
            {
                this.AddMessage(I18nMessage.CR0035, type);
            }
        }
    }
}
