﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that the classes implementing <see cref="IList"/> implements
    /// a strongly typed version of <c>Add</c>, <c>Contains</c>, <c>IndexOf</c>,
    /// <c>Insert</c> and <c>Remove</c> methods.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1039")]
    public class CR0036TypeIListShouldBeStronglyTyped : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="IList"/> type.
        /// </summary>
        private static readonly Type reference = typeof(IList);

        /// <summary>
        /// The names of the methods that should be strongly typed.
        /// </summary>
        private static readonly string[] methods = new string[] { "Add", "Contains", "IndexOf", "Insert", "Remove" };

        /// <summary>
        /// A value indicating whether the parsed type implements
        /// the <see cref="IList"/> interface.
        /// </summary>
        private bool isList;

        /// <summary>
        /// The names of the methods that have been already found in the current
        /// parsed type.
        /// </summary>
        private IList<string> checkedMethods;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0036TypeIListShouldBeStronglyTyped class.
        /// </summary>
        public CR0036TypeIListShouldBeStronglyTyped()
            : base(Constants.Design)
        {
            this.checkedMethods = new List<string>();
        }

        /// <summary>
        /// Initializes the parser to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isList = type.GetInterfaces().Where(i => i == reference).Count() > 0;
            this.checkedMethods.Clear();
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!this.isList)
            {
                return;
            }

            if (method.IsPublic && methods.Contains(method.Name))
            {
                ParameterInfo parameter = method.GetParameters().Where(p => p.Name == "value").FirstOrDefault();
                if (parameter != null && parameter.ParameterType != typeof(object))
                {
                    if (this.checkedMethods.Contains(method.Name))
                    {
                        // The method have been found two times with a good signature
                        string message = string.Format(CultureInfo.CurrentCulture, Error.CR0036DoubleMethod, method.DeclaringType, method.Name);
                        throw new ParserException(message);
                    }

                    this.checkedMethods.Add(method.Name);
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isList && this.checkedMethods.Count != methods.Length)
            {
                foreach (string method in methods.Where(m => !this.checkedMethods.Contains(m)))
                {
                    this.AddMessage(I18nMessage.CR0036, type, method);
                }
            }
        }
    }
}
