﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a type implementing directly the <see cref="IComparable"/>
    /// interface overrides the <see cref="object.Equals(object)"/> method
    /// and the associated operators.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1036")]
    public class CR0037TypeIComparableShouldOverrideEquals : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="IComparable"/> type.
        /// </summary>
        private static readonly Type reference = typeof(IComparable);

        /// <summary>
        /// The names of the expected methods.
        /// </summary>
        private static string[] methods = new string[] { "Equals", "op_Equality", "op_Inequality", "op_GreaterThan", "op_LessThan" };

        /// <summary>
        /// A value indicating whether the parsed type implements
        /// directly <see cref="IComparable"/>.
        /// </summary>
        private bool isComparable;

        /// <summary>
        /// The names of the valid method found in the parsed type.
        /// </summary>
        private IList<string> checkedMethods;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0037TypeIComparableShouldOverrideEquals class.
        /// </summary>
        public CR0037TypeIComparableShouldOverrideEquals()
            : base(Constants.Design)
        {
            this.checkedMethods = new List<string>();
        }

        /// <summary>
        /// Initializes the parser to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isComparable = type.GetInterfaces().Where(i => i == reference).Count() > 0;
            this.checkedMethods.Clear();
            if (type.BaseType != null && type.BaseType.GetInterfaces().Where(i => i == reference).Count() > 0)
            {
                this.isComparable = false;
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!this.isComparable || !methods.Contains(method.Name))
            {
                return;
            }

            ParameterInfo[] parameters = method.GetParameters();
            if (string.Equals(method.Name, "Equals", StringComparison.Ordinal))
            {
                if (method.IsPublic
                    && !method.IsStatic
                    && parameters.Length == 1
                    && parameters[0].ParameterType == typeof(object))
                {
                    this.checkedMethods.Add(method.Name);
                }
            }
            else
            {
                if (method.IsPublic
                    && method.IsStatic
                    && parameters.Length == 2
                    && parameters[0].ParameterType == method.DeclaringType
                    && parameters[1].ParameterType == method.DeclaringType)
                {
                    this.checkedMethods.Add(method.Name);
                }
            }
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isComparable && this.checkedMethods.Count != methods.Length)
            {
                foreach (string method in methods.Where(m => !this.checkedMethods.Contains(m)))
                {
                    this.AddMessage(I18nMessage.CR0037, type, method);
                }
            }
        }
    }
}
