﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks empty interfaces as they can be replaced by attributes.
    /// </summary>
    /// <remarks>
    /// There is no check on nested types as an interface can't declare nested types.
    /// Idem for fields.
    /// </remarks>
    [CheckEquivalence(Constants.FxCop, "CA1040")]
    public class CR0038TypeReplaceEmptyInterfaceWithAttribute : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the current parsed type is an interface.
        /// </summary>
        private bool isInterface;

        /// <summary>
        /// A value indicating whether the interface is considered as empty.
        /// </summary>
        private bool isEmpty;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0038TypeReplaceEmptyInterfaceWithAttribute class.
        /// </summary>
        public CR0038TypeReplaceEmptyInterfaceWithAttribute()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type, property, event and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParsedEvent += this.OnParsedEvent;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isInterface = type.IsInterface;
            this.isEmpty = type.GetInterfaces().Length == 0;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isInterface && this.isEmpty)
            {
                this.AddMessage(I18nMessage.CR0038, type);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!this.isInterface || !this.isEmpty)
            {
                return;
            }

            this.isEmpty = false;
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            if (!this.isInterface || !this.isEmpty)
            {
                return;
            }

            this.isEmpty = false;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!this.isInterface || !this.isEmpty)
            {
                return;
            }

            this.isEmpty = false;
        }
    }
}
