﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the <see cref="ObsoleteAttribute"/> have a message.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1041")]
    public class CR0039AttributeObsoleteShouldHaveMessage : AbstractCheck
    {
        /// <summary>
        /// The reference to the <see cref="ObsoleteAttribute"/> class.
        /// </summary>
        private static readonly Type reference = typeof(ObsoleteAttribute);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0039AttributeObsoleteShouldHaveMessage class.
        /// </summary>
        public CR0039AttributeObsoleteShouldHaveMessage()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type, constructor, field,
        /// event, property and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseConstructorStarted += this.OnParseConstructorStarted;
            parser.ParsedField += this.OnParsedField;
            parser.ParsedEvent += this.OnParsedEvent;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Validates a member.
        /// </summary>
        /// <param name="member">The parsed member.</param>
        /// <returns>
        /// <c>true</c> if the parsed member is not obsolete or has a message.
        /// </returns>
        private static bool IsValid(MemberInfo member)
        {
            CustomAttributeData attribute = member.GetCustomAttributesData()
                .FirstOrDefault(c => c.Constructor.DeclaringType == reference);

            if (attribute == null)
            {
                // No obsolete attribute on the parsed member
                return true;
            }

            if (attribute.ConstructorArguments == null || attribute.ConstructorArguments.Count == 0)
            {
                // Obsolete attribute defined with its default constructor
                return false;
            }

            if (attribute.ConstructorArguments[0].ArgumentType != typeof(string))
            {
                // The two initialized ctor start with the message.
                throw new ParserException(Error.CR0039UnknownObsoleteAttributeConstructor);
            }

            if (string.IsNullOrWhiteSpace(attribute.ConstructorArguments[0].Value as string))
            {
                // Obsolete attribute defined with a null or empty message
                return false;
            }

            return true;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (!IsValid(type))
            {
                this.AddMessage(I18nMessage.CR0039Type, type);
            }
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            if (!IsValid(constructor))
            {
                this.AddMessage(I18nMessage.CR0039Constructor, constructor);
            }
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (!IsValid(field))
            {
                this.AddMessage(I18nMessage.CR0039Field, field);
            }
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            if (!IsValid(eventInfo))
            {
                this.AddMessage(I18nMessage.CR0039Event, eventInfo);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!IsValid(property))
            {
                this.AddMessage(I18nMessage.CR0039Property, property);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!IsValid(method))
            {
                this.AddMessage(I18nMessage.CR0039Method, method);
            }
        }
    }
}
