﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that indexers are based on <see cref="int"/>,
    /// <see cref="long"/> or <see cref="string"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1043")]
    public class CR0040IndexerShouldUseStandardType : AbstractCheck
    {
        /// <summary>
        /// The valid types for indexers.
        /// </summary>
        private static readonly Type[] validTypes = new Type[]
        {
            typeof(int),
            typeof(long),
            typeof(string),
        };

        /// <summary>
        /// Initializes a new instance of the
        /// CR0040IndexerShouldUseStandardType class.
        /// </summary>
        public CR0040IndexerShouldUseStandardType()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on property parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedProperty += this.OnParsedProperty;
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            ParameterInfo[] parameters = property.GetIndexParameters();
            if (parameters != null && parameters.Length > 0)
            {
                bool isValid = true;
                foreach (ParameterInfo parameter in parameters)
                {
                    Type type = parameter.ParameterType;
                    if (!validTypes.Contains(type))
                    {
                        isValid = false;
                    }
                }

                if (!isValid)
                {
                    this.AddMessage(I18nMessage.CR0040, property);
                }
            }
        }
    }
}
