﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a reference type doesn't implements the equality operator
    /// except if add/substract or greater/less operators are defined.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1046")]
    public class CR0042TypeReferenceShouldNotImplementEquality : AbstractCheck
    {
        /// <summary>
        /// The operators that implies an implementation of the Equality operator.
        /// </summary>
        private static string[] operators = new string[] { "op_Addition", "op_Subtraction", "op_GreaterThan", "op_LessThan" };

        /// <summary>
        /// A value indicating whether the parsed type is a reference type.
        /// </summary>
        private bool isReferenceType;

        /// <summary>
        /// A value indicating whether the current type defines at least 
        /// one operator that imply the implementation of the equality operator.
        /// </summary>
        private bool defineOperator;

        /// <summary>
        /// A value indicating whether the paid type implements the equality operator.
        /// </summary>
        private bool defineEquality;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0042TypeReferenceShouldNotImplementEquality class.
        /// </summary>
        public CR0042TypeReferenceShouldNotImplementEquality()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isReferenceType = !type.IsValueType;
            this.defineEquality = false;
            this.defineOperator = false;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.isReferenceType && this.defineEquality && !this.defineOperator)
            {
                this.AddMessage(I18nMessage.CR0042, type);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (operators.Contains(method.Name))
            {
                this.defineOperator = true;
            }
            else if (string.Equals(method.Name, "op_Equality", StringComparison.Ordinal))
            {
                this.defineEquality = true;
            }
        }
    }
}
