﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a sealed type does ot contain virtual members.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1048")]
    public class CR0044TypeSealedShouldNotContainVirtualMember : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the parsed type is sealed.
        /// </summary>
        private bool isSealed;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0044TypeSealedShouldNotContainVirtualMember class.
        /// </summary>
        public CR0044TypeSealedShouldNotContainVirtualMember()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on all type members parsing
        /// excepted field.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseConstructorStarted += this.OnParseConstructorStarted;
            parser.ParsedEvent += this.OnParsedEvent;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isSealed = !typeof(Delegate).IsAssignableFrom(type) && type.IsSealed;
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        /// <remarks>
        /// Pratically, a constructor can never be virtual, but the
        /// reflection framework allows a such situation.
        /// </remarks>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            if (this.isSealed && constructor.IsVirtual)
            {
                this.AddMessage(I18nMessage.CR0044Constructor, constructor);
            }
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            if (!this.isSealed)
            {
                return;
            }

            if (eventInfo.GetAddMethod(true) != null && eventInfo.GetAddMethod(true).HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Event, eventInfo);
            }
            else if (eventInfo.GetRemoveMethod(true) != null && eventInfo.GetRemoveMethod(true).HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Event, eventInfo);
            }
            else if (eventInfo.GetRaiseMethod(true) != null && eventInfo.GetRaiseMethod(true).HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Event, eventInfo);
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!this.isSealed)
            {
                return;
            }

            if (property.CanRead && property.GetGetMethod(true).HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Property, property);
            }
            else if (property.CanWrite && property.GetSetMethod(true).HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Property, property);
            }
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (this.isSealed && method.HasVirtualKeyword())
            {
                this.AddMessage(I18nMessage.CR0044Method, method);
            }
        }
    }
}
