﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a type containing an unmanaged resource implements the
    /// <see cref="IDisposable"/> interface.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1049")]
    public class CR0045TypeWithUnmanagedResourceShouldBeDisposable : AbstractCheck
    {
        /// <summary>
        /// The types associated to unmanaged resources.
        /// </summary>
        private static Type[] unmanagedTypes = new Type[]
        {
            typeof(IntPtr),
            typeof(UIntPtr),
            typeof(HandleRef),
        };

        /// <summary>
        /// A value indicating whether the current type is disposable.
        /// </summary>
        private bool isDisposable;

        /// <summary>
        /// A value indicating whether the parsed type contains unmanaged resources.
        /// </summary>
        private bool containsUnmanagedResources;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0045TypeWithUnmanagedResourceShouldBeDisposable class.
        /// </summary>
        public CR0045TypeWithUnmanagedResourceShouldBeDisposable()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type, field and property parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParsedField += this.OnParsedField;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isDisposable = type.GetInterfaces().FirstOrDefault(t => t == typeof(IDisposable)) != null;
            this.containsUnmanagedResources = false;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.containsUnmanagedResources && !this.isDisposable)
            {
                this.AddMessage(I18nMessage.CR0045, type);
            }
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (unmanagedTypes.Contains(field.FieldType))
            {
                this.containsUnmanagedResources = true;
            }
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (property.IsAutoGenerated() && unmanagedTypes.Contains(property.PropertyType))
            {
                this.containsUnmanagedResources = true;
            }
        }
    }
}
