﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a type containing only static members has no constructor. 
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1053")]
    public class CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructor : AbstractCheck
    {
        /// <summary>
        /// A value indicating whether the parsed type has instance constructors.
        /// </summary>
        private bool hasInstanceConstructors;

        /// <summary>
        /// A value indicating whether the parsed type has instance members.
        /// </summary>
        private bool hasInstanceMembers;

        /// <summary>
        /// A value indicating whether the parsed type has static members.
        /// </summary>
        private bool hasStaticMembers;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructor class.
        /// </summary>
        public CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructor()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type and members parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseTypeEnded += this.OnParseTypeEnded;

            parser.ParseConstructorStarted += this.OnParseConstructorStarted;
            parser.ParsedEvent += this.OnParsedEvent;
            parser.ParsedField += this.OnParsedField;
            parser.ParsedProperty += this.OnParsedProperty;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.hasInstanceConstructors = false;
            this.hasInstanceMembers = false;
            this.hasStaticMembers = false;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            if (this.hasInstanceConstructors && !this.hasInstanceMembers && this.hasStaticMembers)
            {
                this.AddMessage(I18nMessage.CR0049, type);
            }
        }

        /// <summary>
        /// Called when a constructor is parsed.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            // Avoid static and private constructors
            if (constructor.IsStatic || constructor.IsPrivate)
            {
                return;
            }

            this.hasInstanceConstructors = true;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            if (field.IsStatic)
            {
                this.hasStaticMembers = true;
            }
            else
            {
                this.hasInstanceMembers = true;
            }
        }

        /// <summary>
        /// Called when an event is parsed.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            this.OnParseMethodStarted(eventInfo.GetAddMethod());
            this.OnParseMethodStarted(eventInfo.GetRemoveMethod());
            this.OnParseMethodStarted(eventInfo.GetRaiseMethod());
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            this.OnParseMethodStarted(property.GetGetMethod());
            this.OnParseMethodStarted(property.GetSetMethod());
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        /// <remarks>
        /// Because this method is used internally when working on properties or events,
        /// it accepts <c>null</c> value and does nothing in such a case.
        /// </remarks>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (method == null)
            {
                return;
            }

            if (method.IsStatic)
            {
                this.hasStaticMembers = true;
            }
            else
            {
                this.hasInstanceMembers = true;
            }
        }
    }
}
