﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;
using System.Text;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a parameter representing an Uri is associated to <see cref="Uri"/>
    /// and not to <see cref="string"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1054")]
    public class CR0050ParameterUriShouldNotBeString : AbstractCheck
    {
        /// <summary>
        /// The reference words associated to uri elements.
        /// </summary>
        private static readonly string[] references = new string[] { "uri", "url", "urn" };

        /// <summary>
        /// Initializes a new instance of the
        /// CR0050ParameterUriShouldNotBeString class.
        /// </summary>
        public CR0050ParameterUriShouldNotBeString()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Checks if a specific string contains a word associated to Uri.
        /// </summary>
        /// <param name="elementName">
        /// The name of the tested element.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="elementName"/> contains one of the
        /// expected words, <c>false</c> otherwise.
        /// </returns>
        public static bool ContainsUriWord(string elementName)
        {
            StringBuilder builder = new StringBuilder();
            foreach (char c in elementName)
            {
                if (char.IsUpper(c) && builder.Length >= 2)
                {
                    string word = builder.ToString();
                    builder.Clear();

                    if (null != references.FirstOrDefault(w => string.Equals(w, word, StringComparison.OrdinalIgnoreCase)))
                    {
                        return true;
                    }
                }

                builder.Append(c);
            }

            return null != references.FirstOrDefault(w => string.Equals(w, builder.ToString(), StringComparison.OrdinalIgnoreCase));
        }

        /// <summary>
        /// Initializes the parser to react on parameter parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedParameter += this.OnParsedParameter;
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        /// <exception cref="ParserException">
        /// The <paramref name="parameter"/> is associated to a member that is
        /// nor a method neither a constructor.
        /// </exception>
        /// <exception cref="ParserException">
        /// The position of the <paramref name="parameter"/> is greater than
        /// the number of parameters in the associated method.
        /// </exception>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            if (string.IsNullOrEmpty(parameter.Name) || !ContainsUriWord(parameter.Name))
            {
                return;
            }

            if (typeof(string).IsAssignableFrom(parameter.ParameterType))
            {
                MethodBase method = parameter.Member as MethodBase;
                if (method == null)
                {
                    string format = Error.CR0050UnknownMethodType;
                    string message = string.Format(Error.Culture, format, parameter.Member.GetType().FullName);
                    throw new ParserException(message);
                }

                Type[] types = method.GetParameters().Select(p => p.ParameterType).ToArray();
                if (types.Length <= parameter.Position)
                {
                    string format = Error.CR0050Parameters;
                    string message = string.Format(Error.Culture, format, parameter.Member.GetFullName(), types.Length, parameter.Position);
                    throw new ParserException(message);
                }

                Type type = parameter.Member.DeclaringType;
                types[parameter.Position] = typeof(Uri);

                MethodBase associated = null;
                if (method.MemberType == MemberTypes.Method)
                {
                    associated = type.GetMethod(method.Name, Parser.AllDeclaredBindingFlags, null, types, null);
                }
                else if (method.MemberType == MemberTypes.Constructor)
                {
                    associated = type.GetConstructor(Parser.AllDeclaredBindingFlags, null, types, null);
                }
                else
                {
                    string format = Error.CR0050UnknownMethodType;
                    string message = string.Format(Error.Culture, format, parameter.Member.GetType().FullName);
                    throw new ParserException(message);
                }

                if (associated == null)
                {
                    this.AddMessage(I18nMessage.CR0050, method, parameter.Name);
                }
            }
        }
    }
}
