﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks method that hide base method by providing a signature more general than the
    /// original.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1061")]
    public class CR0054MethodShouldNotHideBaseMethod : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0054MethodShouldNotHideBaseMethod class.
        /// </summary>
        public CR0054MethodShouldNotHideBaseMethod()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Retrieve all the methods with a specific name and a specific number
        /// of parameters by recurrence on all the class hierarchy.
        /// </summary>
        /// <param name="type">
        /// The current type.
        /// </param>
        /// <param name="name">
        /// The name of the methods.
        /// </param>
        /// <param name="numberOfParameters">
        /// The number of parameters.
        /// </param>
        /// <returns>
        /// The list of all the methods that respect the crieria of the parameters.
        /// </returns>
        private static ICollection<MethodInfo> ExtractMethods(Type type, string name, int numberOfParameters)
        {
            ICollection<MethodInfo> result;
            if (type.BaseType != null)
            {
                result = ExtractMethods(type.BaseType, name, numberOfParameters);
            }
            else
            {
                result = new LinkedList<MethodInfo>();
            }

            foreach (MethodInfo method in type.GetMethods(Parser.AllDeclaredBindingFlags))
            {
                if (!method.IsPrivate
                    && method.Name == name
                    && method.GetParameters().Length == numberOfParameters)
                {
                    result.Add(method);
                }
            }

            return result;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (method.DeclaringType.BaseType == null)
            {
                // There is no base type
                return;
            }

            int numberOfParameters = method.GetParameters().Length;
            ICollection<MethodInfo> methods = ExtractMethods(method.DeclaringType.BaseType, method.Name, numberOfParameters);
            if (methods == null || methods.Count == 0)
            {
                // There is no base method with the same name and the same number of parameters
                return;
            }

            foreach (MethodInfo baseMethod in methods)
            {
                if (baseMethod.IsDefinitionOf(method))
                {
                    continue;
                }

                for (int i = 0; i < numberOfParameters; i++)
                {
                    Type type = method.GetParameters()[i].ParameterType;
                    Type baseType = baseMethod.GetParameters()[i].ParameterType;

                    if (type != baseType && type.IsAssignableFrom(baseType))
                    {
                        this.AddMessage(I18nMessage.CR0054, method);
                        return;
                    }
                }
            }
        }
    }
}
