﻿using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that native methods are part of a NativeMethods class.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1060")]
    public class CR0056MethodNativeShouldBeGrouped : AbstractCheck
    {
        /// <summary>
        /// The list of the valid type names for native methods.
        /// </summary>
        private static readonly string[] ValidTypeNames = new string[]
        {
            "NativeMethods",
            "SafeNativeMethods",
            "UnsafeNativeMethods",
        };

        /// <summary>
        /// The parsed type can contains native methods.
        /// </summary>
        private bool isValidType;

        /// <summary>
        /// Initializes a new instance of the
        /// CR0056MethodNativeShouldBeGrouped class.
        /// </summary>
        public CR0056MethodNativeShouldBeGrouped()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type and method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.isValidType = ValidTypeNames.Contains(type.Name);
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (method.Attributes.HasFlag(MethodAttributes.PinvokeImpl)
                && !this.isValidType)
            {
                this.AddMessage(I18nMessage.CR0056, method);
            }
        }
    }
}
