﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Implements the code review engine.
    /// </summary>
    public class Engine
    {
        /// <summary>
        /// The registered messages.
        /// </summary>
        private ICollection<Message> messages;

        /// <summary>
        /// The registered checks.
        /// </summary>
        private ICollection<ICheck> checks;

        /// <summary>
        /// The suppressed messages.
        /// </summary>
        private ICollection<IMessage> suppressed;

        /// <summary>
        /// Initializes a new instance of the Engine class.
        /// </summary>
        public Engine()
        {
            this.messages = new LinkedList<Message>();
            this.checks = new LinkedList<ICheck>();
            this.suppressed = new LinkedList<IMessage>();

            this.checks.Add(new ExtractSuppressMessageCheck(this));
        }

        /// <summary>
        /// Gets the registered messages.
        /// </summary>
        public IMessage[] Messages
        {
            get
            {
                // Use of an array to avoid any modification of the collection,
                // Use of contra-variance to expose only the IMessage interface.
                return this.messages.ToArray();
            }
        }

        /// <summary>
        /// Registers a specific check to the engine for the next analysis.
        /// </summary>
        /// <param name="check">
        /// The check to register.
        /// </param>
        public void Register(ICheck check)
        {
            this.checks.Add(check);
        }

        /// <summary>
        /// Executes an analysis on a specific assembly.
        /// </summary>
        /// <param name="assembly">
        /// The analysed assembly.
        /// </param>
        public void DoAnalysis(Assembly assembly)
        {
            // Clear existing results
            this.messages.Clear();
            this.suppressed.Clear();

            // Do the analysis
            IParser parser = new Parser();
            foreach (ICheck check in this.checks)
            {
                check.Initialize(parser, this.AddMessage);
            }

            parser.Parse(assembly);

            // Manage the results
            foreach (Message message in this.messages)
            {
                foreach (IMessage suppressed in this.suppressed)
                {
                    if (MessageComparer.Instance.Equals(message, suppressed))
                    {
                        message.Level = MessageLevel.Ignored;
                    }
                }
            }
        }

        /// <summary>
        /// Adds a message to the registered message of the current analysis.
        /// </summary>
        /// <param name="check">
        /// The current check.
        /// </param>
        /// <param name="localeMessage">
        /// The localized error message.
        /// </param>
        /// <param name="target">
        /// The associated target - could be <c>null</c> if the message targets the assembly.
        /// </param>
        /// <param name="messageId">
        /// The message identifier - could be <c>null</c>.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// The <paramref name="check"/> parameter is <c>null</c>.
        /// </exception>
        /// <exception cref="ArgumentNullException">
        /// One of the <paramref name="localeMessage"/> parameter is <c>null</c> or empty.
        /// </exception>
        [SuppressMessage(
            Constants.Design,
            "CR0026:MethodShouldAvoidRepetitiveParameters",
            Justification = "The parameters are not of the same nature")]
        private void AddMessage(ICheck check, string localeMessage, string target, string messageId)
        {
            if (check == null)
            {
                throw new ArgumentNullException("check");
            }

            if (string.IsNullOrWhiteSpace(localeMessage))
            {
                throw new ArgumentNullException("localeMessage");
            }

            lock (this)
            {
                if (this.messages != null)
                {
                    Message message = new Message();
                    message.Category = check.Category;
                    message.CheckId = check.CheckId;
                    message.Target = target;
                    message.MessageId = messageId;
                    message.LocaleMessage = localeMessage;

                    this.messages.Add(message);
                }
            }
        }

        /// <summary>
        /// Special check that extracts the <see cref="SuppressMessageAttribute"/> from
        /// the parsed assembly.
        /// </summary>
        private class ExtractSuppressMessageCheck : ICheck
        {
            /// <summary>
            /// The current engine.
            /// </summary>
            private readonly Engine engine;

            /// <summary>
            /// Initializes a new instance of the ExtractSuppressMessageCheck class.
            /// </summary>
            /// <param name="engine">
            /// The current engine.
            /// </param>
            public ExtractSuppressMessageCheck(Engine engine)
            {
                if (engine == null)
                {
                    throw new ArgumentNullException("engine");
                }

                this.engine = engine;
            }

            /// <summary>
            /// Gets the category of the check.
            /// </summary>
            public string Category
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets the identifier of the check.
            /// </summary>
            public string CheckId
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Initializes the check for a specific parser.
            /// </summary>
            /// <param name="parser">
            /// The parser associated to the check.
            /// </param>
            /// <param name="addMessage">
            /// The parameter is not used.
            /// </param>
            public void Initialize(IParser parser, AddMessageDelegate addMessage)
            {
                if (object.Equals(this.engine, parser))
                {
                    // TODO: Define the message
                    throw new ArgumentException(string.Empty, "parser");
                }

                parser.ParsedAssemblyAttribute += this.OnParsedAssemblyAttribute;
                parser.ParsedParameter += this.OnParsedParameter;

                parser.ParseConstructorStarted += this.OnParsedAttributeProvider;
                parser.ParsedEvent += this.OnParsedAttributeProvider;
                parser.ParsedField += this.OnParsedAttributeProvider;
                parser.ParsedProperty += this.OnParsedAttributeProvider;
                parser.ParseMethodStarted += this.OnParsedAttributeProvider;
                parser.ParseTypeStarted += this.OnParsedAttributeProvider;
            }

            /// <summary>
            /// Creates and adds a suppressed message to the engine.
            /// </summary>
            /// <param name="attribute">
            /// The meta-data of the reference attribute.
            /// </param>
            /// <param name="target">
            /// The current target - could be <c>null</c>.
            /// </param>
            /// <param name="messageId">
            /// The current message id - could be <c>null</c>.
            /// </param>
            private void TreatAttribute(CustomAttributeData attribute, MemberInfo target, string messageId)
            {
                if (attribute.Constructor.DeclaringType == typeof(SuppressMessageAttribute))
                {
                    IDictionary<string, string> values = new Dictionary<string, string>();
                    values.Add("Category", attribute.ConstructorArguments[0].Value as string);
                    values.Add("CheckId", attribute.ConstructorArguments[1].Value as string);

                    foreach (CustomAttributeNamedArgument arg in attribute.NamedArguments)
                    {
                        values.Add(arg.MemberInfo.Name, arg.TypedValue.Value as string);
                    }

                    Message message = new Message();
                    if (values.ContainsKey("Category"))
                    {
                        message.Category = values["Category"];
                    }

                    if (values.ContainsKey("CheckId"))
                    {
                        message.CheckId = values["CheckId"];
                    }

                    if (values.ContainsKey("Target"))
                    {
                        message.Target = values["Target"];
                    }
                    else
                    {
                        message.Target = target.GetFullName();
                    }

                    if (values.ContainsKey("MessageId"))
                    {
                        message.MessageId = values["MessageId"];
                    }
                    else
                    {
                        message.MessageId = messageId;
                    }

                    this.engine.suppressed.Add(message);
                }
            }

            /// <summary>
            /// Called when an assembly custom attribute is parsed.
            /// </summary>
            /// <param name="attribute">
            /// The parsed attribute.
            /// </param>
            private void OnParsedAssemblyAttribute(CustomAttributeData attribute)
            {
                this.TreatAttribute(attribute, null, null);
            }

            /// <summary>
            /// Called when a member is parsed.
            /// </summary>
            /// <param name="member">
            /// The parsed member.
            /// </param>
            private void OnParsedAttributeProvider(MemberInfo member)
            {
                foreach (CustomAttributeData attribute in member.GetCustomAttributesData())
                {
                    this.TreatAttribute(attribute, member, null);
                }
            }

            /// <summary>
            /// Called when a parameter is parsed.
            /// </summary>
            /// <param name="parameter">
            /// The parsed parameter.
            /// </param>
            private void OnParsedParameter(ParameterInfo parameter)
            {
                foreach (CustomAttributeData attribute in parameter.GetCustomAttributesData())
                {
                    this.TreatAttribute(attribute, parameter.Member, parameter.Name);
                }
            }
        }
    }
}
