﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;

namespace CodeReview.Binary
{
    /// <summary>
    /// Extensions for the <see cref="MemberInfo"/> class.
    /// </summary>
    public static class MemberInfoExtensions
    {
        /// <summary>
        /// The type of the attribute associated with compiler generated elements.
        /// </summary>
        private static readonly ConstructorInfo CompilerGenerated = typeof(CompilerGeneratedAttribute).GetConstructor(Type.EmptyTypes);

        /// <summary>
        /// Checks if an element has a <see cref="CompilerGeneratedAttribute"/> attribute.
        /// </summary>
        /// <param name="member">
        /// The member descriptor.
        /// </param>
        /// <returns>
        /// <c>true</c> if the <paramref name="member"/> has a 
        /// <see cref="CompilerGeneratedAttribute"/> attribute, <c>false</c> otherwise.
        /// </returns>
        public static bool IsCompilerGenerated(this MemberInfo member)
        {
            return null != member.GetCustomAttributesData().FirstOrDefault(a => a.Constructor == CompilerGenerated);
        }

        /// <summary>
        /// Computes the full name associated with a specific member descriptor.
        /// </summary>
        /// <param name="member">
        /// The member descriptor.
        /// </param>
        /// <returns>
        /// The full name of <paramref name="member"/>.
        /// </returns>
        public static string GetFullName(this MemberInfo member)
        {
            // Retrieve the full name of the target
            string result;
            if (member.MemberType == MemberTypes.TypeInfo)
            {
                Type reference = member as Type;
                result = reference.FullName;
            }
            else
            {
                Type reference = member.DeclaringType;
                result = string.Format(CultureInfo.InvariantCulture, "{0}.{1}", reference.FullName, member.Name);
            }

            return result;
        }
    }
}
