﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Extensions for the <see cref="MethodInfo"/> instances.
    /// </summary>
    public static class MethodInfoExtensions
    {
        /// <summary>
        /// Retrieves the definition of the method.
        /// </summary>
        /// <param name="reference">
        /// The refeence method.
        /// </param>
        /// <returns>
        /// The definition of the method.
        /// </returns>
        public static MethodInfo GetDefinition(this MethodInfo reference)
        {
            MethodInfo baseDefinition = reference.GetBaseDefinition();
            if (baseDefinition != reference)
            {
                return baseDefinition;
            }

            foreach (Type type in reference.ReflectedType.GetInterfaces())
            {
                foreach (MethodInfo method in type.GetMethods(Parser.AllDeclaredBindingFlags))
                {
                    if (method.IsDefinitionOf(reference))
                    {
                        return method;
                    }
                }
            }

            return reference;
        }

        /// <summary>
        /// Checks if a method can be considered as the definition of an other.
        /// </summary>
        /// <param name="definition">
        /// The method considered as the definition.
        /// </param>
        /// <param name="method">
        /// The tested method.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="definition"/> is the definition of
        /// <paramref name="method"/>, <c>false</c> otherwise.
        /// </returns>
        public static bool IsDefinitionOf(this MethodInfo definition, MethodInfo method)
        {
            ParameterInfo[] definitionParameters = definition.GetParameters();
            ParameterInfo[] methodParameters = method.GetParameters();

            if (definition.Name != method.Name || definitionParameters.Length != methodParameters.Length)
            {
                return false;
            }
            else
            {
                for (int i = 0; i < definitionParameters.Length; i++)
                {
                    ParameterInfo p = definitionParameters[i];
                    ParameterInfo q = methodParameters[i];
                    if (p.Name != q.Name || p.ParameterType != q.ParameterType)
                    {
                        return false;
                    }
                }

                return true;
            }
        }

        /// <summary>
        /// Checks if the method has been defined as <c>virtual</c>.
        /// </summary>
        /// <param name="method">The checked method.</param>
        /// <returns>
        /// <c>true</c> if the method definition contains the expected elements,
        /// <c>false</c> otherwise.
        /// </returns>
        /// <remarks>
        /// </remarks>
        public static bool HasVirtualKeyword(this MethodInfo method)
        {
            return method.IsVirtual && !method.IsFinal && method.Attributes.HasFlag(MethodAttributes.VtableLayoutMask);
        }
    }
}
