﻿/*
Copyright © 2009-2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Resources;

namespace CodeReview.Binary
{
    /// <summary>
    /// Manages I18n messages through ResX and Enumeration.
    /// </summary>
    internal static class I18nManager
    {
        /// <summary>
        /// The lock objects for the static elements of this class.
        /// </summary>
        private static object lockObject = new object();

        /// <summary>
        /// The resourceManager already loader by the manager.
        /// </summary>
        private static IDictionary<Type, ResourceManager> resourceManagers
            = new Dictionary<Type, ResourceManager>();

        /// <summary>
        /// Gets or sets a <see cref="CultureInfo"/> that overrides the current
        /// thread's CurrentUICulture property for all resource lookups using
        /// this strongly typed resource class.
        /// </summary>
        /// <remarks>
        /// If this property is <c>null</c> (its default value) then the
        /// CurrentUICulture is used to retrieve the resources.
        /// </remarks>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        public static CultureInfo Culture { get; set; }

        /// <summary>
        /// Retrieves a localized string for a specific type and key.
        /// </summary>
        /// <param name="referenceType">
        /// The reference type associated with the resource.
        /// </param>
        /// <param name="key">
        /// The key of the string in the resource manager.
        /// </param>
        /// <returns>
        /// The localized string or <c>null</c> if no localized string exists
        /// for <paramref name="key"/>.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// One of the parameters is <c>null</c>.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// The resource associated with <paramref name="key"/> is not a string.
        /// </exception>
        /// <exception cref="MissingManifestResourceException">
        /// There is no resource file associated with the enumeration type of
        /// <paramref name="referenceType"/>.
        /// </exception>
        public static string GetString(Type referenceType, string key)
        {
            if (referenceType == null)
            {
                throw new ArgumentNullException("referenceType");
            }

            if (string.IsNullOrEmpty(key))
            {
                throw new ArgumentNullException("key");
            }

            lock (lockObject)
            {
                if (!resourceManagers.ContainsKey(referenceType))
                {
                    ResourceManager manager = new ResourceManager(referenceType);
                    resourceManagers.Add(referenceType, manager);
                }

                return resourceManagers[referenceType].GetString(key, Culture);
            }
        }

        /// <summary>
        /// Retrieves a localized string from an enumeration value.
        /// </summary>
        /// <param name="enumValue">
        /// The reference enumeration value.
        /// </param>
        /// <returns>
        /// The localized string or <c>null</c> if no localized string exists
        /// for <paramref name="enumValue"/>.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// The <paramref name="enumValue"/> parameter is <c>null</c>.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// The resource associated with <paramref name="enumValue"/> is not a string.
        /// </exception>
        /// <exception cref="MissingManifestResourceException">
        /// There is no resource file associated with the enumeration type of
        /// <paramref name="enumValue"/>.
        /// </exception>
        public static string GetString(Enum enumValue)
        {
            if (enumValue == null)
            {
                throw new ArgumentNullException("enumValue");
            }

            Type referenceType = enumValue.GetType();
            return GetString(referenceType, enumValue.ToString());
        }

        /// <summary>
        /// Retrieves a localized formatted string from an enumeration value.
        /// </summary>
        /// <param name="enumValue">The reference.</param>
        /// <param name="args">The arguments of the formatted string.</param>
        /// <returns>
        /// The formatted localized string or <c>null</c> if no localized string exists
        /// for <paramref name="enumValue"/>.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// The <paramref name="enumValue"/> parameter is <c>null</c>.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// The resource associated with <paramref name="enumValue"/> is not a string.
        /// </exception>
        /// <exception cref="MissingManifestResourceException">
        /// There is no resource file associated with <paramref name="enumValue"/>.
        /// </exception>
        /// <exception cref="FormatException">
        /// <para>format is invalid.</para>
        /// <para>-or-</para>
        /// <para>The index of a format item is less than zero, or greater than or equal
        /// to the length of the args array.</para>
        /// </exception>
        public static string Format(Enum enumValue, params object[] args)
        {
            string format = GetString(enumValue);
            if (format == null)
            {
                return null;
            }
            else
            {
                return string.Format(Culture, format, args);
            }
        }
    }
}
