﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Diagnostics.CodeAnalysis;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Represents the method that will handle a parsed <see cref="Assembly"/>.
    /// </summary>
    /// <param name="assembly">
    /// The parsed assembly.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void AssemblyHandler(Assembly assembly);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="Attribute"/>.
    /// </summary>
    /// <param name="attribute">
    /// The parsed custom attribute.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void AttributeHandler(CustomAttributeData attribute);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="Type"/>.
    /// </summary>
    /// <param name="type">
    /// The parsed type.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void TypeHandler(Type type);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="FieldInfo"/>.
    /// </summary>
    /// <param name="field">
    /// The parsed field.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void FieldHandler(FieldInfo field);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="ConstructorInfo"/>.
    /// </summary>
    /// <param name="constructor">
    /// The parsed constructor.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void ConstructorHandler(ConstructorInfo constructor);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="EventInfo"/>.
    /// </summary>
    /// <param name="eventInfo">
    /// The parsed event.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void EventHandler(EventInfo eventInfo);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="PropertyInfo"/>.
    /// </summary>
    /// <param name="property">
    /// The parsed property.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void PropertyHandler(PropertyInfo property);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="MethodInfo"/>.
    /// </summary>
    /// <param name="method">
    /// The parsed method.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void MethodHandler(MethodInfo method);

    /// <summary>
    /// Represents the method that will handle a parsed <see cref="ParameterInfo"/>.
    /// </summary>
    /// <param name="parameter">
    /// The parsed parameter.
    /// </param>
    [SuppressMessage(Constants.Design, "CR0012:EventHandlerShouldHaveAProperSignature", Justification = "For the simplification of check writing")]
    public delegate void ParameterHandler(ParameterInfo parameter);

    /// <summary>
    /// Defines the contract of a binary code parser.
    /// </summary>
    public interface IParser
    {
        /// <summary>
        /// Occurs when the parser starts to parse an assembly.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event AssemblyHandler ParseAssemblyStarted;

        /// <summary>
        /// Occurs when the parser ends to parse an assembly.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event AssemblyHandler ParseAssemblyEnded;

        /// <summary>
        /// Occurs when a custom attribute is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event AttributeHandler ParsedAssemblyAttribute;

        /// <summary>
        /// Occurs when the parser starts to parse a type.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event TypeHandler ParseTypeStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a type.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event TypeHandler ParseTypeEnded;

        /// <summary>
        /// Occurs when the parser starts to parse a constructor.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event ConstructorHandler ParseConstructorStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a constructor.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event ConstructorHandler ParseConstructorEnded;

        /// <summary>
        /// Occurs when the parser starts to parse a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event MethodHandler ParseMethodStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event MethodHandler ParseMethodEnded;

        /// <summary>
        /// Occurs when an event is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event EventHandler ParsedEvent;

        /// <summary>
        /// Occurs when a field is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event FieldHandler ParsedField;

        /// <summary>
        /// Occurs when a property is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event PropertyHandler ParsedProperty;

        /// <summary>
        /// Occurs when a parameter of a method or a constructor is parsed
        /// including the return of a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        event ParameterHandler ParsedParameter;

        /// <summary>
        /// Starts the parsing of specific assembly.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        void Parse(Assembly assembly);
    }
}
