﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Diagnostics;

namespace CodeReview.Binary
{
    /// <summary>
    /// Defines the meta-data associated to a code review message.
    /// </summary>
    [DebuggerDisplay("CheckId = <{CheckId}> LocaleMessage = <{LocaleMessage}>")]
    internal sealed class Message : IMessage
    {
        /// <summary>
        /// Initializes a new instance of the Message class.
        /// </summary>
        public Message()
        {
        }

        /// <summary>
        /// Gets or sets the category of the message.
        /// </summary>
        /// <remarks>
        /// By convention, the values of this property is based on the
        /// namespace of the checks-set followed by the code of the category
        /// like <c>CodeReview.Naming</c>.
        /// </remarks>
        public string Category { get; set; }

        /// <summary>
        /// Gets or sets the code associated with the check.
        /// </summary>
        /// <remarks>
        /// <para>
        /// By convention, the code is composed of two letters followed by
        /// two groups of two digits.
        /// </para>
        /// <para>
        /// The letters are associated to the library.
        /// <list type="description">
        /// <item>
        /// <term><c>CR</c></term>
        /// <description>Code Review (this project)</description>
        /// </item>
        /// <item>
        /// <term><c>CA</c></term>
        /// <description>Code Analysis (FxCop)</description>
        /// </item>
        /// <item>
        /// <term><c>SA</c></term>
        /// <description>Source Analysis (SourceCop)</description>
        /// </item>
        /// </list>
        /// </para>
        /// </remarks>
        public string CheckId { get; set; }

        /// <summary>
        /// Gets or sets the full path of the target.
        /// </summary>
        public string Target { get; set; }

        /// <summary>
        /// Gets or sets the identifier of the message.
        /// </summary>
        /// <remarks>
        /// This property is used to give more details about the issue.
        /// </remarks>
        public string MessageId { get; set; }

        /// <summary>
        /// Gets or sets the level of the message.
        /// </summary>
        public MessageLevel Level { get; set; }

        /// <summary>
        /// Gets or sets the localized message associated with the issue.
        /// </summary>
        public string LocaleMessage { get; set; }

        /// <summary>
        /// Returns a string representation of the current instance based on the
        /// <see cref="Message.LocaleMessage"/> property.
        /// </summary>
        /// <returns>
        /// A string representation of the current instance.
        /// </returns>
        public override string ToString()
        {
            return this.LocaleMessage;
        }
    }
}
