﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;

namespace CodeReview.Binary
{
    /// <summary>
    /// Compares instances of the <see cref="IMessage"/> interface.
    /// </summary>
    public sealed class MessageComparer : IEqualityComparer<IMessage>
    {
        /// <summary>
        /// The lock object used to initializes the unique instance of the class.
        /// </summary>
        private static object lockObject = new object();

        /// <summary>
        /// The unique instance of the class.
        /// </summary>
        private static MessageComparer instance;

        /// <summary>
        /// Gets the unique instance of the class.
        /// </summary>
        public static MessageComparer Instance
        {
            get
            {
                lock (lockObject)
                {
                    if (instance == null)
                    {
                        instance = new MessageComparer();
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Compares two instances of the <see cref="IMessage"/> interface.
        /// </summary>
        /// <param name="x">The first instance.</param>
        /// <param name="y">The second instance.</param>
        /// <returns><c>true</c> if the two instance are equal, <c>false</c> otherwise.</returns>
        public bool Equals(IMessage x, IMessage y)
        {
            return string.Equals(x.Category, y.Category, StringComparison.Ordinal)
                && string.Equals(x.CheckId, y.CheckId, StringComparison.Ordinal)
                && string.Equals(x.Target, y.Target, StringComparison.Ordinal)
                && string.Equals(x.MessageId, y.MessageId, StringComparison.Ordinal);
        }

        /// <summary>
        /// Computes the hash code of a <see cref="IMessage"/> instance.
        /// </summary>
        /// <param name="obj">The reference.</param>
        /// <returns>The hash code of the instance.</returns>
        public int GetHashCode(IMessage obj)
        {
            int hash = 0;
            if (obj.Category != null)
            {
                hash = obj.Category.GetHashCode();
            }

            hash = hash << 4;
            if (obj.CheckId != null)
            {
                hash &= obj.CheckId.GetHashCode();
            }

            hash = hash << 4;
            if (obj.Target != null)
            {
                hash &= obj.Target.GetHashCode();
            }

            hash = hash << 4;
            if (obj.MessageId != null)
            {
                hash &= obj.MessageId.GetHashCode();
            }

            return hash;
        }
    }
}
