﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;

namespace CodeReview.Binary
{
    /// <summary>
    /// Default implementation of the <see cref="IParser"/> interface.
    /// </summary>
    internal class Parser : IParser
    {
        /// <summary>
        /// The flags used to retrieve all the declared members.
        /// </summary>
        public const BindingFlags AllDeclaredBindingFlags = BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;

        /// <summary>
        /// The methods starting with one of those prefixes are not parsed.
        /// </summary>
        private static readonly string[] ExcludedMethodPrefixes = new string[] { "add_", "remove_", "get_", "set_" };

        /// <summary>
        /// Occurs when the parser starts to parse an assembly.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event AssemblyHandler ParseAssemblyStarted;

        /// <summary>
        /// Occurs when the parser ends to parse an assembly.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event AssemblyHandler ParseAssemblyEnded;

        /// <summary>
        /// Occurs when a custom attribute is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event AttributeHandler ParsedAssemblyAttribute;

        /// <summary>
        /// Occurs when the parser starts to parse a type.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event TypeHandler ParseTypeStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a type.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event TypeHandler ParseTypeEnded;

        /// <summary>
        /// Occurs when a constructor is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event ConstructorHandler ParseConstructorStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a constructor.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event ConstructorHandler ParseConstructorEnded;

        /// <summary>
        /// Occurs when the parser starts to parse a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event MethodHandler ParseMethodStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event MethodHandler ParseMethodEnded;

        /// <summary>
        /// Occurs when an event is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event EventHandler ParsedEvent;

        /// <summary>
        /// Occurs when a field is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event FieldHandler ParsedField;

        /// <summary>
        /// Occurs when a property is parsed.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event PropertyHandler ParsedProperty;

        /// <summary>
        /// Occurs when a parameter of a method or a constructor is parsed
        /// including the return of a method.
        /// </summary>
        [SuppressMessage(Constants.Design, "CR0005:EventShouldBeSystemEventHandler", Justification = "For the simplification of check writing")]
        public event ParameterHandler ParsedParameter;

        /// <summary>
        /// Starts the parsing of a specific assembly.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        public void Parse(Assembly assembly)
        {
            if (assembly == null)
            {
                throw new ArgumentNullException("assembly");
            }

            if (this.ParseAssemblyStarted != null)
            {
                this.ParseAssemblyStarted(assembly);
            }

            if (this.ParsedAssemblyAttribute != null)
            {
                foreach (CustomAttributeData attribute in assembly.GetCustomAttributesData())
                {
                    this.ParsedAssemblyAttribute(attribute);
                }
            }

            foreach (Type type in assembly.GetTypes())
            {
                // Avoid compiler generated types
                if (type.IsCompilerGenerated())
                {
                    continue;
                }

                this.ParseType(type);
            }

            if (this.ParseAssemblyEnded != null)
            {
                this.ParseAssemblyEnded(assembly);
            }
        }

        /// <summary>
        /// Starts the parsing of a specific type.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        /// <remarks>
        /// The elements of the type are parsed in the following orders:
        /// <list type="bullet">
        /// <item>Fields</item>
        /// <item>Constructors</item>
        /// <item>Events</item>
        /// <item>Properties and Indexers</item>
        /// <item>Methods</item>
        /// </list>
        /// The following elements are not parsed as they are considered as nested types:
        /// <list type="bullet">
        /// <item>Enums</item>
        /// <item>Interfaces</item>
        /// <item>Structs</item>
        /// <item>Classes</item>
        /// </list>
        /// The following elements are not parsed because we should find where they are defined:
        /// <list type="bullet">
        /// <item>Finalizers</item>
        /// <item>Delegates</item>
        /// </list>
        /// </remarks>
        private void ParseType(Type type)
        {
            if (this.ParseTypeStarted != null)
            {
                this.ParseTypeStarted(type);
            }

            // Manage fields
            if (this.ParsedField != null)
            {
                foreach (FieldInfo field in type.GetFields(AllDeclaredBindingFlags))
                {
                    // Avoid compiler generated fields
                    if (field.IsCompilerGenerated())
                    {
                        continue;
                    }

                    // Avoid event fields
                    if (type.GetEvent(field.Name, AllDeclaredBindingFlags) != null)
                    {
                        continue;
                    }

                    this.ParsedField(field);
                }
            }

            // Manage constructors
            ConstructorInfo[] constructors = type.GetConstructors(AllDeclaredBindingFlags);
            if (this.ParseConstructorStarted != null)
            {
                foreach (ConstructorInfo constructor in constructors)
                {
                    this.ParseConstructorStarted(constructor);
                }
            }

            if (this.ParsedParameter != null)
            {
                foreach (ConstructorInfo constructor in constructors)
                {
                    this.ParseParameters(constructor.GetParameters());
                }
            }

            if (this.ParseConstructorEnded != null)
            {
                foreach (ConstructorInfo constructor in constructors)
                {
                    this.ParseConstructorEnded(constructor);
                }
            }

            // Manage events
            if (this.ParsedEvent != null)
            {
                foreach (EventInfo eventInfo in type.GetEvents(AllDeclaredBindingFlags))
                {
                    this.ParsedEvent(eventInfo);
                }
            }

            // Manage properties
            if (this.ParsedProperty != null)
            {
                foreach (PropertyInfo property in type.GetProperties(AllDeclaredBindingFlags))
                {
                    this.ParsedProperty(property);
                }
            }

            // Manage methods
            MethodInfo[] allMethods = type.GetMethods(AllDeclaredBindingFlags);
            IEnumerable<MethodInfo> methods = allMethods.Where(m => ExcludedMethodPrefixes.FirstOrDefault(s => m.Name.StartsWith(s)) == null);

            if (this.ParseMethodStarted != null)
            {
                foreach (MethodInfo method in methods)
                {
                    this.ParseMethodStarted(method);
                }
            }

            if (this.ParsedParameter != null)
            {
                foreach (MethodInfo method in methods)
                {
                    this.ParsedParameter(method.ReturnParameter);
                    this.ParseParameters(method.GetParameters());
                }
            }

            if (this.ParseMethodEnded != null)
            {
                foreach (MethodInfo method in methods)
                {
                    this.ParseMethodEnded(method);
                }
            }

            if (this.ParseTypeEnded != null)
            {
                this.ParseTypeEnded(type);
            }
        }

        /// <summary>
        /// Parses all the parameters of a method or constructor.
        /// </summary>
        /// <param name="parameters">
        /// The collection of parameters to be parsed.
        /// </param>
        private void ParseParameters(ParameterInfo[] parameters)
        {
            if (this.ParsedParameter != null)
            {
                foreach (ParameterInfo parameter in parameters)
                {
                    this.ParsedParameter(parameter);
                }
            }
        }
    }
}
