﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace CodeReview.Command
{
    /// <summary>
    /// Reads the command line parameters and creates a dictionary of key/value pairs.
    /// </summary>
    internal class CommandLineParser
    {
        /// <summary>
        /// The default value of the <see cref="CommandLineParser.ArgumentPrefix"/> property.
        /// </summary>
        public const string DefaultArgumentPrefix = "/";

        /// <summary>
        /// The default value of the <see cref="CommandLineParser.ArgumentAffectation"/> property.
        /// </summary>
        public const string DefaultArgumentAffectation = "=";

        /// <summary>
        /// The default value of the <see cref="CommandLineParser.ValueSeparator"/> property.
        /// </summary>
        public const char DefaultValueSeparator = ',';

        /// <summary>
        /// Initializes a new instance of the CommandLineParser class.
        /// </summary>
        public CommandLineParser()
        {
            this.ArgumentPrefix = DefaultArgumentPrefix;
            this.ArgumentAffectation = DefaultArgumentAffectation;
            this.ValueSeparator = DefaultValueSeparator;
        }

        /// <summary>
        /// Gets or sets the prefix for all commands.
        /// </summary>
        /// <value>
        /// By default, this property is equal to "/".
        /// </value>
        /// <seealso cref="CommandLineParser.DefaultArgumentPrefix" />
        public string ArgumentPrefix { get; set; }

        /// <summary>
        /// Gets or sets the separator used between an argument code and its value.
        /// </summary>
        /// <value>
        /// By default, this property is equal to "=".
        /// </value>
        /// <seealso cref="CommandLineParser.DefaultArgumentAffectation" />
        public string ArgumentAffectation { get; set; }

        /// <summary>
        /// Gets or sets the separator used between values.
        /// </summary>
        /// <value>
        /// By default, this property is equal to ",".
        /// </value>
        /// <seealso cref="CommandLineParser.DefaultValueSeparator" />
        public char ValueSeparator { get; set; }

        /// <summary>
        /// Parses the command line arguments to extract argument/value tokens. 
        /// </summary>
        /// <param name="args">The command line arguments.</param>
        /// <returns>The extracted argument/value tokens.</returns>
        /// <exception cref="CommandException">
        /// One of the argument is not at the right format.
        /// </exception>
        public IDictionary<Option, OptionValues> Tokenize(string[] args)
        {
            IDictionary<Option, OptionValues> result = new Dictionary<Option, OptionValues>();

            string withoutValuePattern = string.Format("^{0}([a-zA-Z\\?\\-_]+)$", this.ArgumentPrefix);
            string withValuePattern = string.Format("^{0}([a-zA-Z\\?\\-_]+){1}(.+)$", this.ArgumentPrefix, this.ArgumentAffectation);
            Regex withoutValue = new Regex(withoutValuePattern);
            Regex withValue = new Regex(withValuePattern);

            bool first = true;
            foreach (string arg in args)
            {
                // Avoid application name
                if (first)
                {
                    if (Environment.CommandLine.StartsWith("\"" + arg))
                    {
                        continue;
                    }

                    first = false;
                }

                string key;
                string[] values;

                // Convert the argument as a code and some values
                Match withoutValueMatch = withoutValue.Match(arg);
                Match withValueMatch = withValue.Match(arg);
                if (withoutValueMatch.Success)
                {
                    key = withoutValueMatch.Groups[1].Value;
                    values = null;
                }
                else if (withValueMatch.Success)
                {
                    key = withValueMatch.Groups[1].Value;
                    values = withValueMatch.Groups[2].Value.Split(this.ValueSeparator);
                }
                else
                {
                    // Issue: Bad formatted parameter
                    string format = Error.CommandLineParserBadFormat;
                    string message = string.Format(Error.Culture, format, arg);

                    throw new CommandException(message);
                }

                // Convert the code to an option
                Option option = OptionFactory.RetrieveOption(key);
                if (option == Option.None)
                {
                    // Issue: Bad option code
                    string format = Error.CommandLineParserBadCode;
                    string message = string.Format(Error.Culture, format, arg, key);

                    throw new CommandException(message);
                }

                // Add the values to the result
                if (!result.ContainsKey(option))
                {
                    result.Add(option, new OptionValues(option));
                }

                if (values != null)
                {
                    result[option].Values.AddAll(values);
                }
            }

            return result;
        }
    }
}
