﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Linq;
using System.Text.RegularExpressions;

namespace CodeReview.Command
{
    /// <summary>
    /// Contains the pre-created <see cref="OptionCodes"/> instances.
    /// </summary>
    public static class OptionFactory
    {
        /// <summary>
        /// The pattern used to valid check identifier.
        /// </summary>
        private const string ValidCheckId = "^[A-Z]{2}[0-9]{4}$";

        /// <summary>
        /// The codes associated to all the options.
        /// </summary>
        private static OptionCodes[] codes = new OptionCodes[]
        {
            new OptionCodes(Option.Assembly, "assembly", "a"),
            new OptionCodes(Option.Exclude, "exclude", "e", "x"),
            new OptionCodes(Option.Include, "include", "i"),
            new OptionCodes(Option.NoConsole, "noconsole", "n"),
            new OptionCodes(Option.Report, "report", "r"),
            new OptionCodes(Option.Help, "help", "h", "?"),
            new OptionCodes(Option.Ignored, "ignored"),
        };

        /// <summary>
        /// The validators associated to all the options.
        /// </summary>
        private static OptionValidator[] validators = new OptionValidator[]
        {
            new OptionValidator(Option.Assembly, OneValue),
            new OptionValidator(Option.Exclude, CheckIdValues),
            new OptionValidator(Option.Include, CheckIdValues),
            new OptionValidator(Option.NoConsole, NoValue),
            new OptionValidator(Option.Report, OneValue),
            new OptionValidator(Option.Help, NoValue),
            new OptionValidator(Option.Ignored, NoValue),
        };

        /// <summary>
        /// Gets the codes associated to all the options.
        /// </summary>
        public static OptionCodes[] Codes
        {
            get { return codes; }
        }

        /// <summary>
        /// Gets the validators associated to all the options.
        /// </summary>
        public static OptionValidator[] Validators
        {
            get { return validators; }
        }

        /// <summary>
        /// Retrieves an option for a specific code.
        /// </summary>
        /// <param name="code">The reference code.</param>
        /// <returns>The associated option or <see cref="Option.None"/>.</returns>
        public static Option RetrieveOption(string code)
        {
            OptionCodes result = Codes.FirstOrDefault(c => c.Codes.Contains(code));
            if (result == null)
            {
                return Option.None;
            }
            else
            {
                return result.Option;
            }
        }

        /// <summary>
        /// Validates that an option has no attached values.  
        /// </summary>
        /// <param name="values">The checked values.</param>
        /// <returns>
        /// <c>true</c> if the <paramref name="values"/> parameter contains
        /// no value.
        /// </returns>
        private static bool NoValue(OptionValues values)
        {
            return values.Values == null || values.Values.Count == 0;
        }

        /// <summary>
        /// Validates that an option has one and only one attached value.  
        /// </summary>
        /// <param name="values">The checked values.</param>
        /// <returns>
        /// <c>true</c> if the <paramref name="values"/> parameter contains
        /// one and only one value.
        /// </returns>
        private static bool OneValue(OptionValues values)
        {
            return values.Values != null || values.Values.Count == 1;
        }

        /// <summary>
        /// Validates that an option has attached values.  
        /// </summary>
        /// <param name="values">The checked values.</param>
        /// <returns>
        /// <c>true</c> if the <paramref name="values"/> parameter contains
        /// values.
        /// </returns>
        private static bool MultiValues(OptionValues values)
        {
            return values.Values != null || values.Values.Count > 0;
        }

        /// <summary>
        /// Validates that an option is associated to valid checkid values.  
        /// </summary>
        /// <param name="values">The checked values.</param>
        /// <returns>
        /// <c>true</c> if the <paramref name="values"/> parameter contains
        /// only valid checkid.
        /// </returns>
        private static bool CheckIdValues(OptionValues values)
        {
            return MultiValues(values) && values.Values.Count(s => !Regex.IsMatch(s, ValidCheckId)) == 0;
        }
    }
}
