﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using CodeReview.Binary;

namespace CodeReview.Command
{
    /// <summary>
    /// Entry point of the application - start a code review process based
    /// on the provided parameter.
    /// </summary>
    internal class Program
    {
        /// <summary>
        /// The actions executed by the program.
        /// </summary>
        private static readonly Action[] actions = new Action[]
        {
            InitValues,
            LoadChecks,
            DoAction,
            CreateReport,
        };

        /// <summary>
        /// The values extracted from the command line arguments.
        /// </summary>
        private static IDictionary<Option, OptionValues> values;

        /// <summary>
        /// The associated engine.
        /// </summary>
        private static Engine engine = new Engine();

        /// <summary>
        /// Starts the application.
        /// </summary>
        public static void Main()
        {
            try
            {
                foreach (Action action in actions)
                {
                    action.Invoke();
                }
            }
            catch (CommandException exception)
            {
                Console.WriteLine(Error.ProgramCommandException, exception.Message);
                Console.WriteLine();
                DisplayHelp();
            }
            catch (Exception exception)
            {
                Console.WriteLine(Error.ProgramException, exception.Message);
                Console.WriteLine();
                DisplayHelp();
            }

#if DEBUG
            Console.ReadKey(true);
#endif
        }

        /// <summary>
        /// Initializes and validates the <see cref="Program.values"/> field.
        /// </summary>
        private static void InitValues()
        {
            CommandLineParser parser = new CommandLineParser();
            values = parser.Tokenize(Environment.GetCommandLineArgs());

            // Basic validation of the values
            foreach (OptionValidator validator in OptionFactory.Validators)
            {
                if (values.ContainsKey(validator.Option))
                {
                    OptionValues value = values[validator.Option];
                    if (!validator.Validator.Invoke(value))
                    {
                        string format = Error.ProgramInvalidValues;
                        string code = OptionFactory.Codes.First(c => c.Option == validator.Option).Codes.First();
                        string message = string.Format(Error.Culture, code, value.Values);
                        throw new CommandException(message);
                    }
                }
            }

            // include or exclude, not both
            if (values.ContainsKey(Option.Include) && values.ContainsKey(Option.Exclude))
            {
                throw new CommandException(Error.ProgramCantHaveIncludeExclude);
            }

            // at least one of help or assembly
            if (!values.ContainsKey(Option.Help) && !values.ContainsKey(Option.Assembly))
            {
                throw new CommandException(Error.ProgramShouldHaveHelpOrAssembly);
            }
        }

        /// <summary>
        /// Loads the checks associated to the current configuration.
        /// </summary>
        private static void LoadChecks()
        {
            if (values.ContainsKey(Option.Include))
            {
                // Manage include
                ICollection<string> includes = values[Option.Include].Values;
                foreach (Type type in typeof(Engine).Assembly.GetTypes())
                {
                    if (type.IsClass && !type.IsAbstract && type.IsPublic && typeof(ICheck).IsAssignableFrom(type))
                    {
                        if (includes.Count(s => type.Name.StartsWith(s)) > 0)
                        {
                            engine.Register((ICheck)Activator.CreateInstance(type));
                        }
                    }
                }
            }
            else
            {
                // Manage exclude and default
                ICollection<string> excludes;
                if (values.ContainsKey(Option.Exclude))
                {
                    excludes = values[Option.Exclude].Values;
                }
                else
                {
                    excludes = new LinkedList<string>();
                }

                foreach (Type type in typeof(Engine).Assembly.GetTypes())
                {
                    if (type.IsClass && !type.IsAbstract && type.IsPublic && typeof(ICheck).IsAssignableFrom(type))
                    {
                        if (excludes.Count(s => type.Name.StartsWith(s)) == 0)
                        {
                            engine.Register((ICheck)Activator.CreateInstance(type));
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Executes the checks.
        /// </summary>
        private static void DoAction()
        {
            if (values.ContainsKey(Option.Help))
            {
                DisplayHelp();
            }
            else if (values.ContainsKey(Option.Assembly))
            {
                foreach (string path in values[Option.Assembly].Values)
                {
                    // TODO: use ReflectionOnlyLoad?
                    string fullPath = Path.GetFullPath(path);
                    Assembly assembly = Assembly.LoadFile(fullPath);
                    engine.DoAnalysis(assembly);
                }
            }
        }

        /// <summary>
        /// Creates the reports about the checks.
        /// </summary>
        private static void CreateReport()
        {
            bool ignored = values.ContainsKey(Option.Ignored);

            // Text report
            if (values.ContainsKey(Option.Report))
            {
                string path = values[Option.Report].Values.First();
                using (TextWriter writer = new StreamWriter(path, false, Encoding.UTF8))
                {
                    WriteMessages(writer, ignored);
                }
            }

            // Console report
            if (!values.ContainsKey(Option.NoConsole))
            {
                WriteMessages(Console.Out, ignored);
            }
        }

        /// <summary>
        /// Writes all the messages on a specific text stream.
        /// </summary>
        /// <param name="writer">
        /// The writer associated to the text-stream.
        /// </param>
        /// <param name="ignored">
        /// A value indicating whether the ignored messages should
        /// be written too.
        /// </param>
        private static void WriteMessages(TextWriter writer, bool ignored)
        {
            foreach (IMessage message in engine.Messages)
            {
                if (ignored || message.Level != MessageLevel.Ignored)
                {
                    writer.WriteLine(
                        "{0} {1} {2} {3} {4}",
                        message.Level,
                        message.Category,
                        message.CheckId,
                        message.Target,
                        message.MessageId);
                }
            }

            writer.WriteLine();
            if (ignored)
            {
                writer.WriteLine(Message.ReportFinish, engine.Messages.Length);
            }
            else
            {
                int count = engine.Messages.Count(m => m.Level != MessageLevel.Ignored);
                writer.WriteLine(Message.ReportFinish, count);
            }
        }

        /// <summary>
        /// Displays the help of the command.
        /// </summary>
        private static void DisplayHelp()
        {
            AssemblyName name = Assembly.GetExecutingAssembly().GetName();
            Console.WriteLine(Message.HelpUsage, name.Name);
            Console.WriteLine(Message.HelpVersion, name.Version);
            Console.WriteLine();

            Console.WriteLine(Message.HelpDetail);
            Console.WriteLine();
        }
    }
}
