﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Linq;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Base class for the unit tests of the checks.
    /// </summary>
    public abstract class AbstractCheckTest
    {
        /// <summary>
        /// The test parser.
        /// </summary>
        private MockParser parser;

        /// <summary>
        /// The last registered message.
        /// </summary>
        private IMessage message;

        /// <summary>
        /// The tested instance.
        /// </summary>
        private ICheck instance;

        /// <summary>
        /// Initializes a new instance of the AbstractCheckTest class.
        /// </summary>
        protected AbstractCheckTest()
        {
            this.parser = new MockParser();
        }

        /// <summary>
        /// Gets the parser used for the tests.
        /// </summary>
        protected MockParser Parser
        {
            get { return this.parser; }
        }

        /// <summary>
        /// Gets the last received message.
        /// </summary>
        /// <remarks>
        /// This property is reinitialized to <c>null</c> before each test.
        /// </remarks>
        protected IMessage Message
        {
            get { return this.message; }
        }

        /// <summary>
        /// Gets the instance of the tested check. 
        /// </summary>
        /// <remarks>
        /// This property is reinitialized before each test by using the
        /// <see cref="AbstractCheckTest.InitializeCheck"/> method.
        /// </remarks>
        protected ICheck Instance
        {
            get { return this.instance; }
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck.CheckId"/> property.
        /// </summary>
        [Test]
        public void CheckId()
        {
            Assert.IsNotNullOrEmpty(this.instance.CheckId);
            string[] parts = this.instance.CheckId.Split(':');
            Assert.AreEqual(2, parts.Length);
            StringAssert.IsMatch("CR[0-9]{4}", parts[0]);
            StringAssert.StartsWith(parts[0] + parts[1], this.instance.GetType().Name);
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck.Category"/> property.
        /// </summary>
        [Test]
        public void Category()
        {
            Assert.AreEqual(Constants.Design, this.instance.Category);
        }

        /// <summary>
        /// Tests that the check is tag with a <see cref="CheckEquivalenceAttribute"/>.
        /// </summary>
        [Test]
        public void HasCheckEquivalence()
        {
            object[] attributes = this.instance.GetType().GetCustomAttributes(typeof(CheckEquivalenceAttribute), false);
            Assert.IsNotNull(attributes);
            Assert.IsNotEmpty(attributes);
            CollectionAssert.AllItemsAreInstancesOfType(attributes, typeof(CheckEquivalenceAttribute));
        }

        /// <summary>
        /// Reinitializes the current check instance used for the tests.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            this.instance = this.InitializeCheck();
            this.message = null;
            this.instance.Initialize(this.parser, this.AddMessage);
        }

        /// <summary>
        /// Clears the last message.
        /// </summary>
        protected void ClearMessage()
        {
            this.message = null;
        }

        /// <summary>
        /// Initializes a new check based on the current tested check.
        /// </summary>
        /// <returns>
        /// An instance of the tested check.
        /// </returns>
        /// <remarks>
        /// This method is used by the <see cref="AbstractCheckTest.SetUp"/>
        /// method to reinitialize the tested check before each test.
        /// </remarks>
        protected abstract ICheck InitializeCheck();

        /// <summary>
        /// The method used to register a new error message.
        /// </summary>
        /// <param name="check">
        /// The current check.
        /// </param>
        /// <param name="localeMessage">
        /// The localized error message.
        /// </param>
        /// <param name="target">
        /// The associated target.
        /// </param>
        /// <param name="messageId">
        /// The message identifier - could be <c>null</c>.
        /// </param>
        private void AddMessage(ICheck check, string localeMessage, string target, string messageId)
        {
            this.message = new Message()
            {
                Category = check.Category,
                CheckId = check.CheckId,
                Target = target,
                MessageId = messageId,
                LocaleMessage = localeMessage,
            };
        }
    }
}
