﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.IO;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0002TypeWithDisposablesShouldBeDisposable"/> class.
    /// </summary>
    [TestFixture]
    public class CR0002TypeWithDisposablesShouldBeDisposableTest : AbstractCheckTest
    {
        /// <summary>
        /// Valids the system requirements of the test to avoid positive-falses.
        /// </summary>
        [Test]
        public static void Prerequirements()
        {
            Assert.IsTrue(typeof(IDisposable).IsAssignableFrom(typeof(Stream)));
            Assert.IsFalse(typeof(IDisposable).IsAssignableFrom(typeof(string)));
        }

        /// <summary>
        /// Tests the <see cref="CR0002TypeWithDisposablesShouldBeDisposable"/> class
        /// on the fields of a not disposable class.
        /// </summary>
        [Test]
        public void CheckNotDisposableField()
        {
            this.Parser.StartParseType(typeof(NotDisposable));
            this.Parser.ParseField(Utilities.GetField(typeof(NotDisposable), "field"));
            this.Parser.EndParseType(typeof(NotDisposable));
            Assert.IsNull(this.Message);

            this.Parser.ParseField(Utilities.GetField(typeof(NotDisposable), "disposableField"));
            this.Parser.EndParseType(typeof(NotDisposable));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the <see cref="CR0002TypeWithDisposablesShouldBeDisposable"/> class
        /// on the properties of a not disposable class.
        /// </summary>
        [Test]
        public void CheckNotDisposableProperty()
        {
            this.Parser.StartParseType(typeof(NotDisposable));
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(NotDisposable), "Property"));
            this.Parser.EndParseType(typeof(NotDisposable));
            Assert.IsNull(this.Message);

            this.Parser.ParseProperty(Utilities.GetProperty(typeof(NotDisposable), "DisposableProperty"));
            this.Parser.EndParseType(typeof(NotDisposable));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the <see cref="CR0002TypeWithDisposablesShouldBeDisposable"/> class
        /// on a disposable class.
        /// </summary>
        [Test]
        public void CheckDisposable()
        {
            this.Parser.StartParseType(typeof(Disposable));
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(Disposable), "Property"));
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(Disposable), "DisposableProperty"));
            this.Parser.ParseField(Utilities.GetField(typeof(Disposable), "field"));
            this.Parser.ParseField(Utilities.GetField(typeof(Disposable), "disposableField"));
            this.Parser.EndParseType(typeof(Disposable));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initiatizes a new instance of the
        /// <see cref="CR0002TypeWithDisposablesShouldBeDisposable"/> class.
        /// </summary>
        /// <returns>
        /// The tested check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0002TypeWithDisposablesShouldBeDisposable();
        }

        /// <summary>
        /// Contains disposable and not disposable members without implementing the 
        /// <see cref="IDisposable"/> interface.
        /// </summary>
        private class NotDisposable
        {
            /// <summary>
            /// A disposable field.
            /// </summary>
            private Stream disposableField;

            /// <summary>
            /// A not disposable field.
            /// </summary>
            private string field;

            /// <summary>
            /// Gets or sets a disposable property.
            /// </summary>
            public Stream DisposableProperty
            {
                get { return this.disposableField; }
                set { this.disposableField = value; }
            }

            /// <summary>
            /// Gets or sets a not disposable property.
            /// </summary>
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }
        }

        /// <summary>
        /// Contains disposable and not disposable members and implements the
        /// <see cref="IDisposable"/> interface.
        /// </summary>
        private class Disposable : IDisposable
        {
            /// <summary>
            /// A disposable field.
            /// </summary>
            private Stream disposableField;

            /// <summary>
            /// A not disposable field.
            /// </summary>
            private string field;

            /// <summary>
            /// Gets or sets a disposable property.
            /// </summary>
            public Stream DisposableProperty
            {
                get { return this.disposableField; }
                set { this.disposableField = value; }
            }

            /// <summary>
            /// Gets or sets a not disposable property.
            /// </summary>
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// Does nothing.
            /// </summary>
            public void Dispose()
            {
            }
        }
    }
}
