﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0004TypeGenericShouldNotContainStaticElement"/> class.
    /// </summary>
    [TestFixture]
    public class CR0004TypeGenericShouldNotContainStaticElementTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a non-generic type.
        /// </summary>
        [Test]
        public void NonGeneric()
        {
            Type reference = typeof(MockClass);
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(reference, "field"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(reference, "staticField"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(reference, "Property"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(reference, "StaticProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(reference, "Method"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(reference, "StaticMethod"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with fields.
        /// </summary>
        [Test]
        public void Field()
        {
            Type reference = typeof(MockGenericClass<>);
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(reference, "field"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(reference, "staticField"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with properties.
        /// </summary>
        [Test]
        public void Property()
        {
            Type reference = typeof(MockGenericClass<>);
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(reference, "Property"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(reference, "StaticProperty"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with methods.
        /// </summary>
        [Test]
        public void Method()
        {
            Type reference = typeof(MockGenericClass<>);
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(reference, "Method"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(reference, "StaticMethod"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initiatizes a new instance of the
        /// <see cref="CR0004TypeGenericShouldNotContainStaticElement"/> class.
        /// </summary>
        /// <returns>
        /// The tested check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0004TypeGenericShouldNotContainStaticElement();
        }

        /// <summary>
        /// A non-generic class used for the tests.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A static field.
            /// </summary>
            private static string staticField;

            /// <summary>
            /// An instance field.
            /// </summary>
            private string field;

            /// <summary>
            /// Gets or sets a static property.
            /// </summary>
            public static string StaticProperty
            {
                get { return staticField; }
                set { staticField = value; }
            }

            /// <summary>
            /// Gets or sets an instance property.
            /// </summary>
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// Defines a static method.
            /// </summary>
            public static void StaticMethod()
            {
            }

            /// <summary>
            /// Defines an instance method.
            /// </summary>
            public void Method()
            {
            }
        }

        /// <summary>
        /// A generic class used for the tests.
        /// </summary>
        /// <typeparam name="T">
        /// A generic parameter.
        /// </typeparam>
        private class MockGenericClass<T>
        {
            /// <summary>
            /// A static field.
            /// </summary>
            private static string staticField;

            /// <summary>
            /// An instance field.
            /// </summary>
            private string field;

            /// <summary>
            /// Gets or sets a static property.
            /// </summary>
            public static string StaticProperty
            {
                get { return staticField; }
                set { staticField = value; }
            }

            /// <summary>
            /// Gets or sets an instance property.
            /// </summary>
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// Defines a static method.
            /// </summary>
            public static void StaticMethod()
            {
            }

            /// <summary>
            /// Defines an instance method.
            /// </summary>
            public void Method()
            {
            }
        }
    }
}
