﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Collections.Generic;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0007MemberShouldNotExposeNestedGenericType"/> class.
    /// </summary>
    [TestFixture]
    public class CR0007MemberShouldNotExposeNestedGenericTypeTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with fields.
        /// </summary>
        [Test]
        public void Field()
        {
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "invalidPrivateField"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "validPrivateField"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "ValidPublicField"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "InvalidPublicField"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with properties.
        /// </summary>
        [Test]
        public void Property()
        {
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "InvalidPrivateGetProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "InvalidPrivateSetProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "InvalidPrivateProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "ValidPrivateProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "ValidPublicGetProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "ValidPublicSetProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "ValidPublicProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "InvalidPublicProperty"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with parameters.
        /// </summary>
        [Test]
        public void Parameter()
        {
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "ValidMethod", "parameter"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "InvalidMethod", "parameter"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with method return type.
        /// </summary>
        [Test]
        public void ReturnType()
        {
            this.Parser.ParseParameter(Utilities.GetMethod(typeof(MockClass), "ValidMethod").ReturnParameter);
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetMethod(typeof(MockClass), "InvalidMethod").ReturnParameter);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0007MemberShouldNotExposeNestedGenericType();
        }

        /// <summary>
        /// Test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// An invalid public field that should failed to the check.
            /// </summary>
            public ICollection<ICollection<string>> InvalidPublicField;

            /// <summary>
            /// A valid public field that should pass the check.
            /// </summary>
            public IList<string> ValidPublicField;

            /// <summary>
            /// An invalid private field that should pass the check.
            /// </summary>
            private ICollection<ICollection<string>> invalidPrivateField;

            /// <summary>
            /// A valid private field that should pass the check.
            /// </summary>
            private IList<string> validPrivateField;

            /// <summary>
            /// Gets a valid public property that should pass the check.
            /// </summary>
            public IList<string> ValidPublicGetProperty
            {
                get { return this.ValidPublicField; }
            }

            /// <summary>
            /// Sets a valid public property that should pass the check.
            /// </summary>
            public IList<string> ValidPublicSetProperty
            {
                set { this.ValidPublicField = value; }
            }

            /// <summary>
            /// Gets or sets a valid public property that should pass the check.
            /// </summary>
            public IList<string> ValidPublicProperty
            {
                get { return this.ValidPublicField; }
                set { this.ValidPublicField = value; }
            }

            /// <summary>
            /// Gets or sets an invalid public property that should failed to the check.
            /// </summary>
            public ICollection<ICollection<string>> InvalidPublicProperty
            {
                get { return this.InvalidPublicField; }
                set { this.InvalidPublicField = value; }
            }

            /// <summary>
            /// Gets an invalid private property that should pass the check.
            /// </summary>
            private ICollection<ICollection<string>> InvalidPrivateGetProperty
            {
                get { return this.invalidPrivateField; }
            }

            /// <summary>
            /// Sets an invalid private property that should pass the check.
            /// </summary>
            private ICollection<ICollection<string>> InvalidPrivateSetProperty
            {
                set { this.invalidPrivateField = value; }
            }

            /// <summary>
            /// Gets or sets an invalid private property that should pass the check.
            /// </summary>
            private ICollection<ICollection<string>> InvalidPrivateProperty
            {
                get { return this.invalidPrivateField; }
                set { this.invalidPrivateField = value; }
            }

            /// <summary>
            /// Gets or sets a valid private property that should pass the check.
            /// </summary>
            private IList<string> ValidPrivateProperty
            {
                get { return this.validPrivateField; }
                set { this.validPrivateField = value; }
            }

            /// <summary>
            /// Exposes a collection implementation on both its return type and its parameter.
            /// </summary>
            /// <param name="parameter">
            /// A parameter that should failed to the check.
            /// </param>
            /// <returns>
            /// A return type that should failed to the check.
            /// </returns>
            public ICollection<ICollection<string>> InvalidMethod(ICollection<ICollection<string>> parameter)
            {
                return parameter;
            }

            /// <summary>
            /// Exposes a collection interface on both its return type and its parameter.
            /// </summary>
            /// <param name="parameter">
            /// A parameter that should pass the check.
            /// </param>
            /// <returns>
            /// A return type that should pass the check.
            /// </returns>
            public IList<string> ValidMethod(IList<string> parameter)
            {
                return parameter;
            }
        }
    }
}
