﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0019AttributeShouldHaveReadOnlyAccessors"/> class.
    /// </summary>
    [TestFixture]
    public class CR0019AttributeShouldHaveReadOnlyAccessorsTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with parameters that are not part of the constructor signature.
        /// </summary>
        [Test]
        public void CheckNamed()
        {
            this.Parser.StartParseType(typeof(MockAttribute));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockAttribute), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockAttribute), "Positioned"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockAttribute), "Named"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with parameters that are part of the constructor signature.
        /// </summary>
        [Test]
        public void CheckPositioned()
        {
            this.Parser.StartParseType(typeof(MockAttribute));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockAttribute), new Type[] { typeof(string), typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockAttribute), "Positioned"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockAttribute), "Named"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with positioned arguments that don't have associated properties.
        /// </summary>
        [Test]
        public void CheckPositionedWithoutProperty()
        {
            this.Parser.StartParseType(typeof(MockAttribute));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockAttribute), new Type[] { typeof(string), typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockAttribute), "Positioned"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockAttribute));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0019AttributeShouldHaveReadOnlyAccessors();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        [AttributeUsage(AttributeTargets.All)]
        private class MockAttribute : Attribute
        {
            /// <summary>
            /// The value of the positioned element.
            /// </summary>
            private readonly string positioned;

            /// <summary>
            /// Initializes a new instance of the MockAttribute class - 
            /// both elements are considered as named elements.
            /// </summary>
            public MockAttribute()
                : this(null, null)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockAttribute class - 
            /// both elements are considered as positioned elements.
            /// </summary>
            /// <param name="positioned">
            /// The value of the positioned element.
            /// </param>
            /// <param name="named">
            /// The value of the named element.
            /// </param>
            public MockAttribute(string positioned, string named)
            {
                this.positioned = positioned;
                this.Named = named;
            }

            /// <summary>
            /// Gets the value of the positioned element.
            /// </summary>
            public string Positioned
            {
                get { return this.positioned; }
            }

            /// <summary>
            /// Gets or sets the value of the named element.
            /// </summary>
            public string Named { get; set; }
        }
    }
}
