﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Reflection.Emit;
using System.Threading;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0021NamespaceShouldHaveMoreThanFourTypes"/> class.
    /// </summary>
    [TestFixture]
    public class CR0021NamespaceShouldHaveMoreThanFourTypesTest : AbstractCheckTest
    {
        /// <summary>
        /// The format of the generated assembly name.
        /// </summary>
        private const string AssemblyFileFormat = "CR0021-assembly-{0}.dll";

        /// <summary>
        /// Tests the check with a good assembly.
        /// </summary>
        [Test]
        public void CheckGood()
        {
            Assembly assembly = GenerateAssembly(0, "MyNamespace.", 5);

            this.Parser.StartParseAssembly(assembly);
            Assert.IsNull(this.Message);
            foreach (Type type in assembly.GetTypes())
            {
                this.Parser.StartParseType(type);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseAssembly(assembly);
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a good assembly using the default namespace.
        /// </summary>
        [Test]
        public void CheckGoodBase()
        {
            Assembly assembly = GenerateAssembly(1, string.Empty, 5);

            this.Parser.StartParseAssembly(assembly);
            Assert.IsNull(this.Message);
            foreach (Type type in assembly.GetTypes())
            {
                this.Parser.StartParseType(type);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseAssembly(assembly);
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a bad assembly.
        /// </summary>
        [Test]
        public void CheckBad()
        {
            Assembly assembly = GenerateAssembly(2, "MyNamespace.", 4);

            this.Parser.StartParseAssembly(assembly);
            Assert.IsNull(this.Message);
            foreach (Type type in assembly.GetTypes())
            {
                this.Parser.StartParseType(type);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseAssembly(assembly);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a bad assembly using the default namespace.
        /// </summary>
        [Test]
        public void CheckBadBase()
        {
            Assembly assembly = GenerateAssembly(3, string.Empty, 4);

            this.Parser.StartParseAssembly(assembly);
            Assert.IsNull(this.Message);
            foreach (Type type in assembly.GetTypes())
            {
                this.Parser.StartParseType(type);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseAssembly(assembly);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0021NamespaceShouldHaveMoreThanFourTypes();
        }

        /// <summary>
        /// Generates a test assembly.
        /// </summary>
        /// <param name="index">
        /// The index of the generated assembly.
        /// </param>
        /// <param name="baseNamespace">
        /// The reference namespace used to build the assembly.
        /// </param>
        /// <param name="numberOfSecondaryTypes">
        /// The number of types to be created in the sub-namespace.
        /// </param>
        /// <returns>
        /// The generated assembly.
        /// </returns>
        private static Assembly GenerateAssembly(int index, string baseNamespace, int numberOfSecondaryTypes)
        {
            string assemblyFileName = string.Format(CultureInfo.InvariantCulture, AssemblyFileFormat, index);
            if (File.Exists(assemblyFileName))
            {
                File.Delete(assemblyFileName);
            }

            AssemblyName myAssemblyName = new AssemblyName();
            myAssemblyName.Name = "CR0021MyAssembly" + index;

            AssemblyBuilder myAssemblyBuilder = Thread.GetDomain().DefineDynamicAssembly(myAssemblyName, AssemblyBuilderAccess.RunAndSave);
            ModuleBuilder myModuleBuilder = myAssemblyBuilder.DefineDynamicModule("CR0021MyModule", assemblyFileName);

            for (int i = 0; i < 3; i++)
            {
                TypeBuilder myTypeBuilder = myModuleBuilder.DefineType(baseNamespace + "MainType" + i);
                myTypeBuilder.CreateType();
            }

            for (int i = 0; i < numberOfSecondaryTypes; i++)
            {
                TypeBuilder myTypeBuilder = myModuleBuilder.DefineType(baseNamespace + "Second.SecondType" + i);
                myTypeBuilder.CreateType();
            }

            myAssemblyBuilder.Save(assemblyFileName);
            return Assembly.ReflectionOnlyLoadFrom(assemblyFileName);
        }
    }
}
