﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0022ParameterShouldNotUseOut"/> class.
    /// </summary>
    [TestFixture]
    public class CR0022ParameterShouldNotUseOutTest : AbstractCheckTest
    {
        /// <summary>
        /// A test interface.
        /// </summary>
        private interface IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="normal">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// A return value.
            /// </returns>
            string FromInterface1(string normal);

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="outParam">
            /// The parameter is not used.
            /// </param>
            void FromInterface2(out string outParam);
        }

        /// <summary>
        /// Tests the check on interface members.
        /// </summary>
        [Test]
        public void CheckInterface()
        {
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(IMockInterface), "FromInterface1", "normal"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(IMockInterface), "FromInterface1", string.Empty));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(IMockInterface), "FromInterface2", "outParam"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on abstract class members.
        /// </summary>
        [Test]
        public void CheckAbstract()
        {
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(AbstractMockClass), "FromAbstract", "outParam"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on class members.
        /// </summary>
        [Test]
        public void CheckClass()
        {
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "FromInterface1", "normal"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "FromInterface1", string.Empty));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "FromInterface2", "outParam"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "FromAbstract", "outParam"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method1", "outParam"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on members fragged as ref instead of out.
        /// </summary>
        [Test]
        public void CheckByRef()
        {
           this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method2", "refParam"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0022ParameterShouldNotUseOut();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private abstract class AbstractMockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="outParam">
            /// The parameter is not used.
            /// </param>
            public abstract void FromAbstract(out string outParam);
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass : AbstractMockClass, IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="normal">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// A return value.
            /// </returns>
            public string FromInterface1(string normal)
            {
                return normal.Clone() as string;
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="outParam">
            /// The parameter is not used.
            /// </param>
            public void FromInterface2(out string outParam)
            {
                outParam = "test";
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="outParam">
            /// The parameter is not used.
            /// </param>
            public override void FromAbstract(out string outParam)
            {
                outParam = "test";
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="outParam">
            /// The parameter is not used.
            /// </param>
            public void Method1(out string outParam)
            {
                outParam = "test";
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="refParam">
            /// The parameter is not used.
            /// </param>
            public void Method2(ref string refParam)
            {
                refParam = "test";
            }
        }
    }
}
