﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0026MethodShouldAvoidRepetitiveParameters"/> class.
    /// </summary>
    [TestFixture]
    public class CR0026MethodShouldAvoidRepetitiveParametersTest : AbstractCheckTest
    {
        /// <summary>
        /// A test method definition.
        /// </summary>
        /// <param name="p1">The parameter is not used.</param>
        /// <param name="p2">The parameter is not used.</param>
        /// <param name="p3">The parameter is not used.</param>
        public delegate void Delegate3(string p1, string p2, string p3);

        /// <summary>
        /// A test method definition.
        /// </summary>
        /// <param name="p1">The parameter is not used.</param>
        /// <param name="p2">The parameter is not used.</param>
        /// <param name="p3">The parameter is not used.</param>
        /// <param name="p4">The parameter is not used.</param>
        public delegate void Delegate4(string p1, string p2, string p3, string p4);

        /// <summary>
        /// Tests the check on methods.
        /// </summary>
        [Test]
        public void CheckMethod()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "Method1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "Method2"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "Method3"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GoodMethod"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "Method4"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on delegates.
        /// </summary>
        [Test]
        public void CheckDelegate()
        {
            this.Parser.StartParseType(typeof(Delegate3));
            Assert.IsNull(this.Message);
            this.ClearMessage();

            foreach (MethodInfo method in typeof(Delegate3).GetMethods())
            {
                this.Parser.ParseMethod(method);
                Assert.IsNull(this.Message);
            }

            this.Parser.StartParseType(typeof(Delegate4));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(Delegate4), "Invoke"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0026MethodShouldAvoidRepetitiveParameters();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private abstract class MockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            public abstract void Method1(string p1);

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            public abstract void Method2(string p1, string p2);

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <param name="p3">The parameter is not used.</param>
            public abstract void Method3(string p1, string p2, string p3);

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <param name="p3">The parameter is not used.</param>
            /// <param name="p4">The parameter is not used.</param>
            public abstract void Method4(string p1, string p2, string p3, string p4);

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <param name="p3">The parameter is not used.</param>
            /// <param name="p4">The parameter is not used.</param>
            public abstract void GoodMethod(string p1, string p2, string p3, params string[] p4);
        }
    }
}
