﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0027EnumShouldUseFlagsAttribute"/> class.
    /// </summary>
    [TestFixture]
    public class CR0027EnumShouldUseFlagsAttributeTest : AbstractCheckTest
    {
        /// <summary>
        /// A good candidate for a Flags attribute.
        /// </summary>
        private enum MockCandidateEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 4,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// A good candidate for a Flags attribute.
        /// </summary>
        private enum MockCandidateEnumLong : ulong
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 4,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// A bad candidate for a Flags attribute.
        /// </summary>
        private enum MockEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 3,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// A test flag.
        /// </summary>
        [Flags]
        private enum MockFlags
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 4,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// Tests the check on a existing flags.
        /// </summary>
        /// <param name="type">
        /// The tested type of enumeration.
        /// </param>
        /// <param name="expectedMessage">
        /// A value indicating whether the test should expect a message
        /// from the check for this <paramref name="type"/>.
        /// </param>
        [TestCase(typeof(MockFlags), false, Description = "On an existing flags")]
        [TestCase(typeof(MockEnum), false, Description = "On a classic enumeration")]
        [TestCase(typeof(MockCandidateEnum), true, Description = "On a good candidate enumeration")]
        [TestCase(typeof(MockCandidateEnumLong), true, Description = "On a good candidate enumeration with long storage")]
        public void CheckFlag(Type type, bool expectedMessage)
        {
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);
            foreach (FieldInfo field in type.GetFields())
            {
                this.Parser.ParseField(field);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(type);
            if (expectedMessage)
            {
                Assert.IsNotNull(this.Message);
            }
            else
            {
                Assert.IsNull(this.Message);
            }
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0027EnumShouldUseFlagsAttribute();
        }
    }
}
