﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0028EnumShouldNotUseFlagsAttribute"/> class.
    /// </summary>
    [TestFixture]
    public class CR0028EnumShouldNotUseFlagsAttributeTest : AbstractCheckTest
    {
        /// <summary>
        /// A classic enumeration.
        /// </summary>
        private enum MockEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 3,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// A bad flags declaration.
        /// </summary>
        [Flags]
        private enum MockBadFlags
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 3,

            /// <summary>
            /// A test value.
            /// </summary>
            All = Second | Third,
        }

        /// <summary>
        /// A good flags declaration.
        /// </summary>
        [Flags]
        private enum MockGoodFlags
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            First = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Second = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Third = 4,

            /// <summary>
            /// A test value.
            /// </summary>
            All = First | Second | Third,
        }

        /// <summary>
        /// Tests the check on an enumeration.
        /// </summary>
        [Test]
        public void CheckEnum()
        {
            this.Parser.StartParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
            foreach (FieldInfo field in typeof(MockEnum).GetFields())
            {
                this.Parser.ParseField(field);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a bad flags declaration.
        /// </summary>
        [Test]
        public void CheckBadFlags()
        {
            this.Parser.StartParseType(typeof(MockBadFlags));
            Assert.IsNull(this.Message);
            foreach (FieldInfo field in typeof(MockBadFlags).GetFields())
            {
                this.Parser.ParseField(field);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(MockBadFlags));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a good flags declaration.
        /// </summary>
        [Test]
        public void CheckGoodFlags()
        {
            this.Parser.StartParseType(typeof(MockGoodFlags));
            Assert.IsNull(this.Message);
            foreach (FieldInfo field in typeof(MockGoodFlags).GetFields())
            {
                this.Parser.ParseField(field);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(MockGoodFlags));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0028EnumShouldNotUseFlagsAttribute();
        }
    }
}
