﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Runtime.Serialization;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0031TypeExceptionShouldHaveStandardConstructors"/> class.
    /// </summary>
    [TestFixture]
    public class CR0031TypeExceptionShouldHaveStandardConstructorsTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with a valid sealed exception.
        /// </summary>
        [Test]
        public void CheckValidSealed()
        {
            this.Parser.StartParseType(typeof(MockSealedException));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockSealedException), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockSealedException), new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockSealedException), new Type[] { typeof(string), typeof(Exception) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockSealedException), new Type[] { typeof(SerializationInfo), typeof(StreamingContext) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockSealedException));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a valid exception with more defined constructors.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(MockValidException));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockValidException), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockValidException), new Type[] { typeof(int) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockValidException), new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockValidException), new Type[] { typeof(string), typeof(Exception) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockValidException), new Type[] { typeof(SerializationInfo), typeof(StreamingContext) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockValidException));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with missing constructors.
        /// </summary>
        [Test]
        public void CheckMissing()
        {
            this.Parser.StartParseType(typeof(MockInvalidException));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(string), typeof(Exception) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(SerializationInfo), typeof(StreamingContext) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockInvalidException));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a protected constructor.
        /// </summary>
        [Test]
        public void CheckVisibility()
        {
            this.Parser.StartParseType(typeof(MockInvalidException));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(string), typeof(Exception) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockInvalidException), new Type[] { typeof(SerializationInfo), typeof(StreamingContext) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockInvalidException));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0031TypeExceptionShouldHaveStandardConstructors();
        }

        /// <summary>
        /// A sealed exception valid for the check.
        /// </summary>
        public sealed class MockSealedException : Exception
        {
            /// <summary>
            /// Initializes a new instance of the MockSealedException class.
            /// </summary>
            public MockSealedException()
                : base()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockSealedException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            public MockSealedException(string message)
                : base(message)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockSealedException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            /// <param name="innerException">The parameter is not used.</param>
            public MockSealedException(string message, Exception innerException)
                : base(message, innerException)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockSealedException class.
            /// </summary>
            /// <param name="info">The parameter is not used.</param>
            /// <param name="context">The parameter is not used.</param>
            private MockSealedException(SerializationInfo info, StreamingContext context)
                : base(info, context)
            {
            }
        }

        /// <summary>
        /// An exception valid for the check.
        /// </summary>
        public class MockValidException : Exception
        {
            /// <summary>
            /// Initializes a new instance of the MockValidException class.
            /// </summary>
            public MockValidException()
                : base()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockValidException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            public MockValidException(string message)
                : base(message)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockValidException class.
            /// </summary>
            /// <param name="parameter">The parameter is not used.</param>
            public MockValidException(int parameter)
                : base()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockValidException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            /// <param name="innerException">The parameter is not used.</param>
            public MockValidException(string message, Exception innerException)
                : base(message, innerException)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockValidException class.
            /// </summary>
            /// <param name="info">The parameter is not used.</param>
            /// <param name="context">The parameter is not used.</param>
            protected MockValidException(SerializationInfo info, StreamingContext context)
                : base(info, context)
            {
            }
        }

        /// <summary>
        /// An invalid exception for the check.
        /// </summary>
        public class MockInvalidException : Exception
        {
            /// <summary>
            /// Initializes a new instance of the MockInvalidException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            public MockInvalidException(string message)
                : base(message)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockInvalidException class.
            /// </summary>
            /// <param name="message">The parameter is not used.</param>
            /// <param name="innerException">The parameter is not used.</param>
            public MockInvalidException(string message, Exception innerException)
                : base(message, innerException)
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockInvalidException class.
            /// </summary>
            protected MockInvalidException()
                : base()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockInvalidException class.
            /// </summary>
            /// <param name="info">The parameter is not used.</param>
            /// <param name="context">The parameter is not used.</param>
            protected MockInvalidException(SerializationInfo info, StreamingContext context)
                : base(info, context)
            {
            }
        }
    }
}
