﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0032MethodExplicitShouldBeOverridable"/> class.
    /// </summary>
    [TestFixture]
    public class CR0032MethodExplicitShouldBeOverridableTest : AbstractCheckTest
    {
        /// <summary>
        /// The flags used to extract methods.
        /// </summary>
        private static BindingFlags flags = CodeReview.Binary.Parser.AllDeclaredBindingFlags;

        /// <summary>
        /// A test interface.
        /// </summary>
        public interface IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            void Method();
        }

        /// <summary>
        /// Tests the check on a good classes.
        /// </summary>
        /// <param name="className">
        /// The name of the tested nested class.
        /// </param>
        [TestCase("MockGoodClassPublic")]
        [TestCase("MockGoodClassPrivate")]
        public void CheckGoodClass(string className)
        {
            Type type = this.GetType().Assembly.GetType(this.GetType().FullName + "+" + className);
            Assert.IsNotNull(type);

            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);

            bool inLoop = false;
            foreach (MethodInfo method in type.GetMethods(flags))
            {
                inLoop = true;
                this.Parser.StartParseMethod(method);
                Assert.IsNull(this.Message);
            }

            Assert.IsTrue(inLoop);
        }

        /// <summary>
        /// Tests the check on bad classes.
        /// </summary>
        /// <param name="className">
        /// The name of the tested nested class.
        /// </param>
        [TestCase("MockBadClassWithVisibleMethod")]
        [TestCase("MockBadClassWithPrivateMethod")]
        [TestCase("MockBadClassWithStaticMethod")]
        public void CheckBadClass(string className)
        {
            Type type = this.GetType().Assembly.GetType(this.GetType().FullName + "+" + className);
            Assert.IsNotNull(type);
            
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);

            bool inLoop = false;
            foreach (MethodInfo method in type.GetMethods(flags))
            {
                inLoop = true;
                if (method.Name.Contains("."))
                {
                    this.Parser.StartParseMethod(method);
                    Assert.IsNotNull(this.Message);
                    this.ClearMessage();
                }
            }

            Assert.IsTrue(inLoop);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0032MethodExplicitShouldBeOverridable();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        public class MockBadClassWithVisibleMethod : IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            void IMockInterface.Method()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        public class MockBadClassWithPrivateMethod : IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            void IMockInterface.Method()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            private void Method()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        public class MockBadClassWithStaticMethod : IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            public static void Method()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            void IMockInterface.Method()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        public class MockGoodClassPublic : IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            void IMockInterface.Method()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            protected virtual void Method()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockGoodClassPrivate : IMockInterface
        {
            /// <summary>
            /// A test method.
            /// </summary>
            void IMockInterface.Method()
            {
            }
        }
    }
}
