﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0036TypeIListShouldBeStronglyTyped"/> class.
    /// </summary>
    [TestFixture]
    public class CR0036TypeIListShouldBeStronglyTypedTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with a valid list.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(MockList));
            Assert.IsNull(this.Message);

            foreach (MethodInfo method in typeof(MockList).GetMethods(BindingFlags.Instance | BindingFlags.Public | BindingFlags.DeclaredOnly))
            {
                this.Parser.StartParseMethod(method);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(MockList));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with an invalid list - one of the method will use object
        /// instead of string element.
        /// </summary>
        /// <param name="methodName">The name of the method with wrong parameters.</param>
        [TestCase("Add")]
        [TestCase("Contains")]
        [TestCase("IndexOf")]
        [TestCase("Insert")]
        [TestCase("Remove")]
        public void CheckInvalid(string methodName)
        {
            this.Parser.StartParseType(typeof(MockList));
            Assert.IsNull(this.Message);

            Type[] addTypes = string.Equals(methodName, "Add") ? new Type[] { typeof(object) } : new Type[] { typeof(string) };
            Type[] containsTypes = string.Equals(methodName, "Contains") ? new Type[] { typeof(object) } : new Type[] { typeof(string) };
            Type[] indexOfTypes = string.Equals(methodName, "IndexOf") ? new Type[] { typeof(object) } : new Type[] { typeof(string) };
            Type[] insertTypes = string.Equals(methodName, "Insert") ? new Type[] { typeof(int), typeof(object) } : new Type[] { typeof(int), typeof(string) };
            Type[] removeTypes = string.Equals(methodName, "Remove") ? new Type[] { typeof(object) } : new Type[] { typeof(string) };

            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockList), "Add", addTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockList), "Contains", containsTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockList), "IndexOf", indexOfTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockList), "Insert", insertTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockList), "Remove", removeTypes));
            Assert.IsNull(this.Message);

            this.Parser.EndParseType(typeof(MockList));
            Assert.IsNotNull(this.Message);
            Assert.AreEqual(methodName, this.Message.MessageId);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0036TypeIListShouldBeStronglyTyped();
        }

        /// <summary>
        /// A test implementation of the <see cref="IList"/> interface.
        /// </summary>
        private class MockList : IList
        {
            /// <summary>
            /// Gets the count of element - not used.
            /// </summary>
            public int Count
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a value indicating whether the list is synchronized - not used.
            /// </summary>
            public bool IsSynchronized
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets the synchronization object - not used.
            /// </summary>
            public object SyncRoot
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a value indicating whether the list has a fixed length - not used.
            /// </summary>
            public bool IsFixedSize
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a value indicating whether the list can't be updated - not used.
            /// </summary>
            public bool IsReadOnly
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a specific element of the list - not used.
            /// </summary>
            /// <param name="index">The index of the element.</param>
            /// <returns>The associated element.</returns>
            public object this[int index]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// This method is not used.
            /// </summary>
            public void Clear()
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// This method is not used.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            public void RemoveAt(int index)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// This method is not used.
            /// </summary>
            /// <param name="array">The parameter is not used.</param>
            /// <param name="index">The parameter is not used.</param>
            public void CopyTo(Array array, int index)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// This method is not used.
            /// </summary>
            /// <returns>Always raise an exception.</returns>
            public IEnumerator GetEnumerator()
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Default implementation of the <see cref="IList.Add"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public int Add(object value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Default implementation of the <see cref="IList.Contains"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public bool Contains(object value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Default implementation of the <see cref="IList.IndexOf"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public int IndexOf(object value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Default implementation of the <see cref="IList.Insert"/> method.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <param name="value">The parameter is not used.</param>
            public void Insert(int index, object value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Default implementation of the <see cref="IList.Remove"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            public void Remove(object value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Strongly-typed implementation of the <see cref="IList.Add"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public int Add(string value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Strongly-typed implementation of the <see cref="IList.Contains"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public bool Contains(string value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Strongly-typed implementation of the <see cref="IList.IndexOf"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public int IndexOf(string value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Strongly-typed implementation of the <see cref="IList.Insert"/> method.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <param name="value">The parameter is not used.</param>
            public void Insert(int index, string value)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// Strongly-typed implementation of the <see cref="IList.Remove"/> method.
            /// </summary>
            /// <param name="value">The parameter is not used.</param>
            public void Remove(string value)
            {
                throw new NotImplementedException();
            }
        }
    }
}
