﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0037TypeIComparableShouldOverrideEquals"/> class.
    /// </summary>
    [TestFixture]
    public class CR0037TypeIComparableShouldOverrideEqualsTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a valid class.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(MockComparable));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "CompareTo"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "Equals"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "GetHashCode"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_Equality"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_Inequality"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_GreaterThan"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_LessThan"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockComparable));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check when one of the expected method is missing.
        /// </summary>
        /// <param name="missing">The name of the missing method or operator.</param>
        [TestCase("Equals")]
        [TestCase("op_Equality")]
        [TestCase("op_Inequality")]
        [TestCase("op_GreaterThan")]
        [TestCase("op_LessThan")]
        public void CheckMissing(string missing)
        {
            this.Parser.StartParseType(typeof(MockComparable));
            Assert.IsNull(this.Message);

            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "CompareTo"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "GetHashCode"));
            Assert.IsNull(this.Message);

            if (!string.Equals(missing, "Equals"))
            {
                this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "Equals"));
                Assert.IsNull(this.Message);
            }

            if (!string.Equals(missing, "op_Equality"))
            {
                this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_Equality"));
                Assert.IsNull(this.Message);
            }

            if (!string.Equals(missing, "op_Inequality"))
            {
                this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_Inequality"));
                Assert.IsNull(this.Message);
            }

            if (!string.Equals(missing, "op_GreaterThan"))
            {
                this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_GreaterThan"));
                Assert.IsNull(this.Message);
            }

            if (!string.Equals(missing, "op_LessThan"))
            {
                this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockComparable), "op_LessThan"));
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(MockComparable));
            Assert.IsNotNull(this.Message);
            Assert.AreEqual(missing, this.Message.MessageId);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0037TypeIComparableShouldOverrideEquals();
        }

        /// <summary>
        /// A test class implementing <see cref="IComparable"/>.
        /// </summary>
        private class MockComparable : IComparable
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator ==(MockComparable p1, MockComparable p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator !=(MockComparable p1, MockComparable p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator >(MockComparable p1, MockComparable p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator <(MockComparable p1, MockComparable p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="obj">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public int CompareTo(object obj)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="obj">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public override bool Equals(object obj)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>Always raise an exception.</returns>
            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }
    }
}
