﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0038TypeReplaceEmptyInterfaceWithAttribute"/> class.
    /// </summary>
    [TestFixture]
    public class CR0038TypeReplaceEmptyInterfaceWithAttributeTest : AbstractCheckTest
    {
        /// <summary>
        /// A test interface.
        /// </summary>
        private interface IMockInterface
        {
            /// <summary>
            /// A test event.
            /// </summary>
            event EventHandler<EventArgs> Event;

            /// <summary>
            /// Gets a test property.
            /// </summary>
            string Property { get; }

            /// <summary>
            /// A test method.
            /// </summary>
            void Method();
        }

        /// <summary>
        /// A test interface.
        /// </summary>
        private interface IMockChildInterface : ICloneable
        {
        }

        /// <summary>
        /// Tests the check on a class.
        /// </summary>
        [Test]
        public void CheckEmptyClass()
        {
            this.Parser.StartParseType(typeof(MockEmptyClass));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockEmptyClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an empty interface.
        /// </summary>
        [Test]
        public void CheckEmptyInterface()
        {
            this.Parser.StartParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(IMockInterface));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an non-empty interface.
        /// </summary>
        [Test]
        public void CheckInterfaceWithProperty()
        {
            this.Parser.StartParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(IMockInterface), "Property"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an non-empty interface.
        /// </summary>
        [Test]
        public void CheckInterfaceWithEvent()
        {
            this.Parser.StartParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(IMockInterface), "Event"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an non-empty interface.
        /// </summary>
        [Test]
        public void CheckInterfaceWithMethod()
        {
            this.Parser.StartParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(IMockInterface), "Method"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(IMockInterface));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an non-empty interface.
        /// </summary>
        [Test]
        public void CheckInterfaceWithChildInterface()
        {
            this.Parser.StartParseType(typeof(IMockChildInterface));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(IMockChildInterface));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0038TypeReplaceEmptyInterfaceWithAttribute();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockEmptyClass
        {
        }
    }
}
