﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0039AttributeObsoleteShouldHaveMessage"/> class.
    /// </summary>
    [TestFixture]
    public class CR0039AttributeObsoleteShouldHaveMessageTest : AbstractCheckTest
    {
        /// <summary>
        /// Validates the status of the <see cref="ObsoleteAttribute"/> class
        /// to ensure the accuraccy of the unit-test.
        /// </summary>
        [Test]
        public void CheckTheTest()
        {
            AttributeTargets expected = AttributeTargets.Class | AttributeTargets.Struct | AttributeTargets.Enum | AttributeTargets.Constructor | AttributeTargets.Method | AttributeTargets.Property | AttributeTargets.Field | AttributeTargets.Event | AttributeTargets.Interface | AttributeTargets.Delegate;
            AttributeUsageAttribute[] attributes = typeof(ObsoleteAttribute).GetCustomAttributes(typeof(AttributeUsageAttribute), false) as AttributeUsageAttribute[];
            Assert.IsNotEmpty(attributes);
            Assert.AreEqual(1, attributes.Length);
            Assert.AreEqual(expected, attributes[0].ValidOn);
        }

        /// <summary>
        /// Tests the check on valid Obsolete attributes.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            Type reference = this.GetType().Assembly.GetType(this.GetType().FullName + "+MockValidClass");
            Assert.IsNotNull(reference);

            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(reference, Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(reference, "Method"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(reference, "Property"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(reference, "field"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(reference, "Event"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on invalid Obsolete attributes.
        /// </summary>
        [Test]
        public void CheckInvalid()
        {
            Type reference = this.GetType().Assembly.GetType(this.GetType().FullName + "+MockInvalidClass");
            Assert.IsNotNull(reference);

            this.Parser.StartParseType(reference);
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseConstructor(Utilities.GetConstructor(reference, Type.EmptyTypes));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseMethod(Utilities.GetMethod(reference, "Method"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseProperty(Utilities.GetProperty(reference, "Property"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseField(Utilities.GetField(reference, "field"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseEvent(Utilities.GetEvent(reference, "Event"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0039AttributeObsoleteShouldHaveMessage();
        }

        /// <summary>
        /// An obsolete test class with messages.
        /// </summary>
        [Obsolete("with message")]
        private class MockValidClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            [Obsolete("with message")]
            private string field;

            /// <summary>
            /// Initializes a new instance of the MockValidClass class.
            /// </summary>
            [Obsolete("with message")]
            public MockValidClass()
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            [Obsolete("with message")]
            public event EventHandler<EventArgs> Event;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            [Obsolete("with message")]
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            [Obsolete("with message")]
            public void Method()
            {
            }
        }

        /// <summary>
        /// An obsolete test class with messages.
        /// </summary>
        [Obsolete()]
        private class MockInvalidClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            [Obsolete()]
            private string field;

            /// <summary>
            /// Initializes a new instance of the MockInvalidClass class.
            /// </summary>
            [Obsolete()]
            public MockInvalidClass()
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            [Obsolete()]
            public event EventHandler<EventArgs> Event;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            [Obsolete()]
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            [Obsolete()]
            public void Method()
            {
            }
        }
    }
}
