﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0042TypeReferenceShouldNotImplementEquality"/> class.
    /// </summary>
    [TestFixture]
    public class CR0042TypeReferenceShouldNotImplementEqualityTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a classic reference type.
        /// </summary>
        [Test]
        public void CheckClassic()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a reference type with only equality operator.
        /// </summary>
        [Test]
        public void CheckEqualityAlone()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "op_Equality"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a reference type with equality and difference operator.
        /// </summary>
        [Test]
        public void CheckEquality()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "op_Equality"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "op_Inequality"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a reference type with equality and math operators.
        /// </summary>
        /// <param name="operatorName">The name of the math operator to load.</param>
        [TestCase("op_Addition")]
        [TestCase("op_Subtraction")]
        [TestCase("op_GreaterThan")]
        [TestCase("op_LessThan")]
        public void CheckOperator(string operatorName)
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "op_Equality"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), operatorName));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0042TypeReferenceShouldNotImplementEquality();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator ==(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator !=(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator <(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static bool operator >(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static MockClass operator +(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="p1">The parameter is not used.</param>
            /// <param name="p2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public static MockClass operator -(MockClass p1, MockClass p2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="obj">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public override bool Equals(object obj)
            {
                return base.Equals(obj);
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>Always raise an exception.</returns>
            public override int GetHashCode()
            {
                throw new NotImplementedException();
            }
        }
    }
}
