﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0043TypeSealedShouldNotContainProtectedMember"/> class.
    /// </summary>
    [TestFixture]
    public class CR0043TypeSealedShouldNotContainProtectedMemberTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on public or private elements.
        /// </summary>
        /// <param name="type">
        /// The tested type.
        /// </param>
        [TestCase(typeof(MockClass), Description = "on normal class")]
        [TestCase(typeof(MockSealedClass), Description = "on sealed class")]
        public void CheckNonProtected(Type type)
        {
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(type, "privateField"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(type, Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(type, "PublicEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(type, "PublicProperty"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(type, "PublicMethod"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on protected elements in a normal class.
        /// </summary>
        [Test]
        public void CheckProtectedOnClassic()
        {
            Type type = typeof(MockClass);
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(type, "protectedField"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(type, new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(type, "ProtectedEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(type, "ProtectedProperty"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(type, "ProtectedMethod"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on protected elements in a sealed class.
        /// </summary>
        [Test]
        public void CheckProtectedOnSealed()
        {
            Type type = typeof(MockSealedClass);
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);

            this.Parser.ParseField(Utilities.GetField(type, "protectedField"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseConstructor(Utilities.GetConstructor(type, new Type[] { typeof(string) }));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseEvent(Utilities.GetEvent(type, "ProtectedEvent"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseProperty(Utilities.GetProperty(type, "ProtectedProperty"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseMethod(Utilities.GetMethod(type, "ProtectedMethod"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0043TypeSealedShouldNotContainProtectedMember();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            protected string protectedField;

            /// <summary>
            /// A test field.
            /// </summary>
            private string privateField;

            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            public MockClass()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            /// <param name="p">The parameter is not used.</param>
            protected MockClass(string p)
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler<EventArgs> PublicEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            protected event EventHandler<EventArgs> ProtectedEvent;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string PublicProperty
            {
                get { return this.privateField; }
                set { this.privateField = value; }
            }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            protected string ProtectedProperty
            {
                get { return this.protectedField; }
                set { this.protectedField = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public void PublicMethod()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            protected void ProtectedMethod()
            {
            }
        }

        /// <summary>
        /// A sealed test class.
        /// </summary>
        private sealed class MockSealedClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            protected string protectedField;

            /// <summary>
            /// A test field.
            /// </summary>
            private string privateField;

            /// <summary>
            /// Initializes a new instance of the MockSealedClass class.
            /// </summary>
            public MockSealedClass()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockSealedClass class.
            /// </summary>
            /// <param name="p">The parameter is not used.</param>
            protected MockSealedClass(string p)
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler<EventArgs> PublicEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            protected event EventHandler<EventArgs> ProtectedEvent;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string PublicProperty
            {
                get { return this.privateField; }
                set { this.privateField = value; }
            }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            protected string ProtectedProperty
            {
                get { return this.protectedField; }
                set { this.protectedField = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public void PublicMethod()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            protected void ProtectedMethod()
            {
            }
        }
    }
}
