﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0044TypeSealedShouldNotContainVirtualMember"/> class.
    /// </summary>
    [TestFixture]
    public class CR0044TypeSealedShouldNotContainVirtualMemberTest : AbstractCheckTest
    {
        /// <summary>
        /// A test interface.
        /// </summary>
        private interface IMockInterface
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            string InterfaceProperty { get; set; }
        }

        /// <summary>
        /// Tests the check on public or private elements.
        /// </summary>
        /// <param name="type">
        /// The tested type.
        /// </param>
        [TestCase(typeof(MockClass), Description = "on normal class")]
        [TestCase(typeof(MockSealedClass), Description = "on sealed class")]
        public void CheckNonVirtual(Type type)
        {
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(type, Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "Event"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "Property"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "InterfaceProperty"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Method"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on protected elements in a normal class.
        /// </summary>
        [Test]
        public void CheckVirtualOnClassic()
        {
            Type type = typeof(MockClass);
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(type, "VirtualEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(type, "VirtualProperty"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(type, "VirtualMethod"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on protected elements in a sealed class.
        /// </summary>
        [Test]
        public void CheckVirtualOnSealed()
        {
            Type type = typeof(MockSealedClass);
            this.Parser.StartParseType(type);
            Assert.IsNull(this.Message);

            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "VirtualEvent"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "VirtualProperty"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "VirtualMethod"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0044TypeSealedShouldNotContainVirtualMember();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass : IMockInterface
        {
            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            public MockClass()
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler<EventArgs> Event;

            /// <summary>
            /// A test event.
            /// </summary>
            public virtual event EventHandler<EventArgs> VirtualEvent;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string InterfaceProperty { get; set; }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string Property { get; set; }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public virtual string VirtualProperty { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Method()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public virtual void VirtualMethod()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private sealed class MockSealedClass : MockClass
        {
            /// <summary>
            /// Initializes a new instance of the MockSealedClass class.
            /// </summary>
            public MockSealedClass()
            {
            }
        }
    }
}
