﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Runtime.InteropServices;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0045TypeWithUnmanagedResourceShouldBeDisposable"/> class.
    /// </summary>
    [TestFixture]
    public class CR0045TypeWithUnmanagedResourceShouldBeDisposableTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with a disposable class that contains unmanaged resources.
        /// </summary>
        [Test]
        public void CheckValidDisposable()
        {
            this.Parser.StartParseType(typeof(MockDisposableClass));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockDisposableClass), "unmanagedField"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockDisposableClass), "UnmanagedProperty"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockDisposableClass), "ManagedProperty"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockDisposableClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a not disposable class that doesn't contain unmanaged resources.
        /// </summary>
        [Test]
        public void CheckValidNotDisposable()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "ManagedProperty"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a not disposable class that contains a unmanaged resources.
        /// </summary>
        [Test]
        public void CheckInvalidField()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "unmanagedField"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a not disposable class that contains a unmanaged resources.
        /// </summary>
        [Test]
        public void CheckInvalidProperty()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "UnmanagedProperty"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0045TypeWithUnmanagedResourceShouldBeDisposable();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            private IntPtr unmanagedField;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public HandleRef UnmanagedProperty { get; set; }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public IntPtr ManagedProperty
            {
                get { return this.unmanagedField; }
                set { this.unmanagedField = value; }
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockDisposableClass : IDisposable
        {
            /// <summary>
            /// A test field.
            /// </summary>
            private IntPtr unmanagedField;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public HandleRef UnmanagedProperty { get; set; }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public IntPtr ManagedProperty
            {
                get { return this.unmanagedField; }
                set { this.unmanagedField = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Dispose()
            {
                throw new NotImplementedException();
            }
        }
    }
}
