﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.IO;
using System.Reflection;
using System.Reflection.Emit;
using System.Threading;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0046TypeShouldBeDefinedInANamespace"/> class.
    /// </summary>
    [TestFixture]
    public class CR0046TypeShouldBeDefinedInANamespaceTest : AbstractCheckTest
    {
        /// <summary>
        /// The name of the generated assembly.
        /// </summary>
        private const string WrongAssemblyFileName = "CR0046-MockAssembly.exe";

        /// <summary>
        /// Creates the wrong assembly.
        /// </summary>
        /// <remarks>
        /// Based on a MSDN sample:
        /// http://msdn.microsoft.com/fr-fr/library/system.reflection.assemblyname.name.aspx
        /// </remarks>
        [SetUp]
        public static void WrongAssemblySetUp()
        {
            if (File.Exists(WrongAssemblyFileName))
            {
                File.Delete(WrongAssemblyFileName);
            }

            AssemblyName myAssemblyName = new AssemblyName();
            myAssemblyName.Name = "CR0046MyAssembly";

            AssemblyBuilder myAssemblyBuilder = Thread.GetDomain().DefineDynamicAssembly(myAssemblyName, AssemblyBuilderAccess.RunAndSave);
            ModuleBuilder myModuleBuilder = myAssemblyBuilder.DefineDynamicModule("CR0046MyModule", WrongAssemblyFileName);
            TypeBuilder myTypeBuilder = myModuleBuilder.DefineType("MyType");
            MethodBuilder myMethodBuilder = myTypeBuilder.DefineMethod("Main", MethodAttributes.Public | MethodAttributes.HideBySig | MethodAttributes.Static, typeof(void), null);
            ILGenerator myILGenerator = myMethodBuilder.GetILGenerator();
            myILGenerator.EmitWriteLine("Hello World!");
            myILGenerator.Emit(OpCodes.Ret);
            myTypeBuilder.CreateType();
            myAssemblyBuilder.SetEntryPoint(myMethodBuilder);

            myAssemblyBuilder.Save(WrongAssemblyFileName);
        }

        /// <summary>
        /// Deletes the wrong assembly.
        /// </summary>
        [TearDown]
        public static void WrongAssemblyTearDown()
        {
            if (File.Exists(WrongAssemblyFileName))
            {
                File.Delete(WrongAssemblyFileName);
            }
        }

        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            Assembly testAssembly = Assembly.ReflectionOnlyLoadFrom(WrongAssemblyFileName);
            Assert.IsNotNull(testAssembly);

            Type testType = testAssembly.GetType("MyType");
            Assert.IsNotNull(testType);

            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);

            this.Parser.StartParseType(testType);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0046TypeShouldBeDefinedInANamespace();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
        }
    }
}
