﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructor"/> class.
    /// </summary>
    [TestFixture]
    public class CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructorTest : AbstractCheckTest
    {
        /// <summary>
        /// The test classes used during the tests.
        /// </summary>
        private static readonly Type[] mockClasses = new Type[]
        {
            typeof(MockClass),
            typeof(MockSealedClass),
            typeof(MockStaticClass),
        };

        /// <summary>
        /// The instance members used by the tests.
        /// </summary>
        private static readonly MemberInfo[] instanceMembers = new MemberInfo[]
        {
            Utilities.GetField(typeof(MockClass), "field"),
            Utilities.GetEvent(typeof(MockClass), "Event"),
            Utilities.GetProperty(typeof(MockClass), "Property"),
            Utilities.GetMethod(typeof(MockClass), "Method"),
        };

        /// <summary>
        /// The static members used by the tests.
        /// </summary>
        private static readonly MemberInfo[] staticMembers = new MemberInfo[]
        {
            Utilities.GetField(typeof(MockClass), "staticField"),
            Utilities.GetEvent(typeof(MockClass), "StaticEvent"),
            Utilities.GetProperty(typeof(MockClass), "StaticProperty"),
            Utilities.GetMethod(typeof(MockClass), "StaticMethod"),
        };

        /// <summary>
        /// Tests the check with empty types.
        /// </summary>
        /// <param name="reference">
        /// The tested type.
        /// </param>
        [TestCase(typeof(MockClass), Description = "non sealed class")]
        [TestCase(typeof(MockSealedClass), Description = "sealed class")]
        [TestCase(typeof(MockStaticClass), Description = "static class")]
        public void CheckEmptyType(Type reference)
        {
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(reference);
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with types that contains an instance member.
        /// </summary>
        /// <param name="reference">
        /// The tested type.
        /// </param>
        /// <param name="member">
        /// The tested member.
        /// </param>
        [Test]
        [Combinatorial]
        public void CheckInstance(
            [ValueSource("mockClasses")]Type reference,
            [ValueSource("instanceMembers")]MemberInfo member)
        {
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockClass), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.ParseMember(member);
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(reference);
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with types that contains a static member.
        /// </summary>
        /// <param name="reference">
        /// The tested type.
        /// </param>
        /// <param name="member">
        /// The tested member.
        /// </param>
        [Test]
        [Combinatorial]
        public void CheckStatic(
            [ValueSource("mockClasses")]Type reference,
            [ValueSource("staticMembers")]MemberInfo member)
        {
            this.Parser.StartParseType(reference);
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockClass), Type.EmptyTypes));
            Assert.IsNull(this.Message);
            this.Parser.ParseMember(member);
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(reference);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0049TypeWithOnlyStaticMemberShouldNotHaveConstructor();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private static class MockStaticClass
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test field.
            /// </summary>
            private static string staticField;

            /// <summary>
            /// A test field.
            /// </summary>
            private string field;

            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            public MockClass()
            {
            }

            /// <summary>
            /// A test event.
            /// </summary>
            public static event EventHandler<EventArgs> StaticEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler<EventArgs> Event;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public static string StaticProperty
            {
                get { return staticField; }
                set { staticField = value; }
            }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string Property
            {
                get { return this.field; }
                set { this.field = value; }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public static void StaticMethod()
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Method()
            {
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private sealed class MockSealedClass
        {
        }
    }
}
