﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.IO;
using System.Reflection;
using System.Reflection.Emit;
using System.Threading;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0055TypeExceptionShouldBePublic"/> class.
    /// </summary>
    [TestFixture]
    public class CR0055TypeExceptionShouldBePublicTest : AbstractCheckTest
    {
        /// <summary>
        /// The name of the generated assembly.
        /// </summary>
        private const string AssemblyFileName = "CR0055-MockAssembly.exe";

        /// <summary>
        /// Creates the assembly.
        /// </summary>
        /// <remarks>
        /// Based on a MSDN sample: http://msdn.microsoft.com/fr-fr/library/system.reflection.assemblyname.name.aspx
        /// </remarks>
        [SetUp]
        public static void AssemblySetUp()
        {
            if (File.Exists(AssemblyFileName))
            {
                File.Delete(AssemblyFileName);
            }

            AssemblyName myAssemblyName = new AssemblyName();
            myAssemblyName.Name = "CR0055MyAssembly";

            AssemblyBuilder myAssemblyBuilder = Thread.GetDomain().DefineDynamicAssembly(myAssemblyName, AssemblyBuilderAccess.RunAndSave);
            ModuleBuilder myModuleBuilder = myAssemblyBuilder.DefineDynamicModule("CR0055MyModule", AssemblyFileName);
            TypeBuilder myTypeBuilder = myModuleBuilder.DefineType("MockPublicException", TypeAttributes.Public | TypeAttributes.Class, typeof(Exception));
            myTypeBuilder.CreateType();
            myAssemblyBuilder.Save(AssemblyFileName);
        }

        /// <summary>
        /// Deletes the assembly.
        /// </summary>
        [TearDown]
        public static void AssemblyTearDown()
        {
            if (File.Exists(AssemblyFileName))
            {
                File.Delete(AssemblyFileName);
            }
        }

        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            Assembly assembly = Assembly.ReflectionOnlyLoadFrom(AssemblyFileName);
            this.Parser.StartParseType(assembly.GetType("MockPublicException"));
            Assert.IsNull(this.Message);

            this.Parser.StartParseType(typeof(MockPublicClass));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.StartParseType(typeof(MockPrivateClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0055TypeExceptionShouldBePublic();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockPublicClass : Exception
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockPrivateClass : Exception
        {
        }
    }
}
