﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the CodeReview.Binary assembly.
    /// </summary>
    [TestFixture]
    public static class GlobalTest
    {
        /// <summary>
        /// All the public methods of the test classes should be associated
        /// to a unit-test and should have a TestAttribute.
        /// </summary>
        [Test]
        public static void MissingTestAttribute()
        {
            foreach (Type type in typeof(GlobalTest).Assembly.GetTypes())
            {
                if (type.GetCustomAttributesData().FirstOrDefault(a => a.Constructor.DeclaringType.Name == "TestFixtureAttribute") == null)
                {
                    continue;
                }

                foreach (MethodInfo method in type.GetMethods(BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.Instance))
                {
                    if (method.GetCustomAttributesData().FirstOrDefault(a => a.Constructor.DeclaringType.Namespace == "NUnit.Framework") == null)
                    {
                        Assert.Fail("The method <{0}.{1}> should have a Test attribute", type.FullName, method.Name);
                    }
                }
            }
        }

        /// <summary>
        /// All the public classes should be unit-test classes.
        /// </summary>
        [Test]
        public static void MissingTestFixtureAttribute()
        {
            foreach (Type type in typeof(GlobalTest).Assembly.GetTypes())
            {
                if (type.IsPublic
                    && !type.IsInterface
                    && !type.IsAbstract
                    && type.Name.EndsWith("Test", StringComparison.Ordinal)
                    && type.GetCustomAttributesData().FirstOrDefault(a => a.Constructor.DeclaringType.Name == "TestFixtureAttribute") == null)
                {
                    Assert.Fail("The type <{0}> should have a TestFixture attribute", type.FullName);
                }
            }
        }

        /// <summary>
        /// Runs the engine on the CodeReview.Binary assembly.
        /// </summary>
        [Test]
        public static void RunEngine()
        {
            Type reference = typeof(ICheck);
            Assembly assembly = reference.Assembly;
            Engine engine = new Engine();

            bool withAtLeastOneCheck = false;
            foreach (Type type in assembly.GetTypes())
            {
                if (type.IsPublic && !type.IsAbstract && !type.IsInterface && reference.IsAssignableFrom(type))
                {
                    engine.Register((ICheck)Activator.CreateInstance(type));
                    withAtLeastOneCheck = true;
                }
            }

            Assert.IsTrue(withAtLeastOneCheck);

            engine.DoAnalysis(assembly);

            Assert.IsNotNull(engine.Messages);

            // Test that all the messages are associated to ignored element
            CollectionAssert.IsEmpty(engine.Messages.Where(m => m.Level != MessageLevel.Ignored).ToArray());
        }

        /// <summary>
        /// Tests that all the checks have unique identifier.
        /// </summary>
        [Test]
        public static void UniqueCheckId()
        {
            Type reference = typeof(ICheck);
            Assembly assembly = reference.Assembly;
            IDictionary<string, Type> tested = new Dictionary<string, Type>();

            bool withAtLeastOneCheck = false;
            foreach (Type type in assembly.GetTypes())
            {
                if (type.IsPublic && !type.IsAbstract && !type.IsInterface && reference.IsAssignableFrom(type))
                {
                    withAtLeastOneCheck = true;
                    ICheck instance = (ICheck)Activator.CreateInstance(type);
                    string[] split = instance.CheckId.Split(':');

                    // Tests the result of the split without Assert, this is part of
                    // individual check tests.
                    if (split.Length == 2)
                    {
                        string shortId = split[0];
                        if (tested.ContainsKey(shortId))
                        {
                            Assert.Fail(
                                "The check id <{0}> is defined for both <{1}> and <{2}>",
                                shortId,
                                tested[shortId].Name,
                                type.Name);
                        }

                        tested.Add(shortId, type);
                    }
                }
            }

            Assert.IsTrue(withAtLeastOneCheck);
        }
    }
}
