﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Parser used as part of the checks tests.
    /// </summary>
    public class MockParser : IParser
    {
        /// <summary>
        /// Occurs when the parser starts to parse an assembly.
        /// </summary>
        public event AssemblyHandler ParseAssemblyStarted;

        /// <summary>
        /// Occurs when the parser ends to parse an assembly.
        /// </summary>
        public event AssemblyHandler ParseAssemblyEnded;

        /// <summary>
        /// Occurs when a custom attribute is parsed.
        /// </summary>
        public event AttributeHandler ParsedAssemblyAttribute;

        /// <summary>
        /// Occurs when the parser starts to parse a type.
        /// </summary>
        public event TypeHandler ParseTypeStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a type.
        /// </summary>
        public event TypeHandler ParseTypeEnded;

        /// <summary>
        /// Occurs when a constructor is parsed.
        /// </summary>
        public event ConstructorHandler ParseConstructorStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a constructor.
        /// </summary>
        public event ConstructorHandler ParseConstructorEnded;

        /// <summary>
        /// Occurs when the parser starts to parse a method.
        /// </summary>
        public event MethodHandler ParseMethodStarted;

        /// <summary>
        /// Occurs when the parser ends to parse a method.
        /// </summary>
        public event MethodHandler ParseMethodEnded;

        /// <summary>
        /// Occurs when a event is parsed.
        /// </summary>
        public event EventHandler ParsedEvent;

        /// <summary>
        /// Occurs when a field is parsed.
        /// </summary>
        public event FieldHandler ParsedField;

        /// <summary>
        /// Occurs when a property is parsed.
        /// </summary>
        public event PropertyHandler ParsedProperty;

        /// <summary>
        /// Occurs when a parameter of a method or a constructor is parsed
        /// including the return of a method.
        /// </summary>
        public event ParameterHandler ParsedParameter;

        /// <summary>
        /// This method is not implemented.
        /// </summary>
        /// <param name="assembly">
        /// This parameter is not used.
        /// </param>
        /// <exception cref="NotImplementedException">
        /// This method always raises this exception.
        /// </exception>
        public void Parse(Assembly assembly)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Parses the member with the associated ParseXxx method.
        /// </summary>
        /// <param name="member">
        /// The parsed element.
        /// </param>
        public void ParseMember(MemberInfo member)
        {
            if (member.MemberType == MemberTypes.Field)
            {
                this.ParseField((FieldInfo)member);
            }
            else if (member.MemberType == MemberTypes.Event)
            {
                this.ParseEvent((EventInfo)member);
            }
            else if (member.MemberType == MemberTypes.Property)
            {
                this.ParseProperty((PropertyInfo)member);
            }
            else if (member.MemberType == MemberTypes.Method)
            {
                this.ParseMethod((MethodInfo)member);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseAssemblyStarted"/> event.
        /// </summary>
        /// <param name="assembly">
        /// The parsed element.
        /// </param>
        public void StartParseAssembly(Assembly assembly)
        {
            if (this.ParseAssemblyStarted != null)
            {
                this.ParseAssemblyStarted(assembly);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseAssemblyEnded"/> event.
        /// </summary>
        /// <param name="assembly">
        /// The parsed element.
        /// </param>
        public void EndParseAssembly(Assembly assembly)
        {
            if (this.ParseAssemblyEnded != null)
            {
                this.ParseAssemblyEnded(assembly);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParsedAssemblyAttribute"/> event.
        /// </summary>
        /// <param name="attribute">
        /// The parsed element.
        /// </param>
        public void ParseAssemblyAttribute(CustomAttributeData attribute)
        {
            if (this.ParsedAssemblyAttribute != null)
            {
                this.ParsedAssemblyAttribute(attribute);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseTypeStarted"/> event.
        /// </summary>
        /// <param name="type">
        /// The parsed element.
        /// </param>
        public void StartParseType(Type type)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }

            if (this.ParseTypeStarted != null)
            {
                this.ParseTypeStarted(type);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseTypeEnded"/> event.
        /// </summary>
        /// <param name="type">
        /// The parsed element.
        /// </param>
        public void EndParseType(Type type)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }

            if (this.ParseTypeEnded != null)
            {
                this.ParseTypeEnded(type);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseConstructorStarted"/> event.
        /// </summary>
        /// <param name="constructor">
        /// The parsed element.
        /// </param>
        public void StartParseConstructor(ConstructorInfo constructor)
        {
            if (constructor == null)
            {
                throw new ArgumentNullException("constructor");
            }

            if (this.ParseConstructorStarted != null)
            {
                this.ParseConstructorStarted(constructor);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseConstructorEnded"/> event.
        /// </summary>
        /// <param name="constructor">
        /// The parsed element.
        /// </param>
        public void EndParseConstructor(ConstructorInfo constructor)
        {
            if (constructor == null)
            {
                throw new ArgumentNullException("constructor");
            }

            if (this.ParseConstructorEnded != null)
            {
                this.ParseConstructorEnded(constructor);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseMethodStarted"/> event.
        /// </summary>
        /// <param name="method">
        /// The parsed element.
        /// </param>
        public void StartParseMethod(MethodInfo method)
        {
            if (method == null)
            {
                throw new ArgumentNullException("method");
            }

            if (this.ParseMethodStarted != null)
            {
                this.ParseMethodStarted(method);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParseMethodEnded"/> event.
        /// </summary>
        /// <param name="method">
        /// The parsed element.
        /// </param>
        public void EndParseMethod(MethodInfo method)
        {
            if (method == null)
            {
                throw new ArgumentNullException("method");
            }

            if (this.ParseMethodEnded != null)
            {
                this.ParseMethodEnded(method);
            }
        }

        /// <summary>
        /// Fires both <see cref="MockParser.ParseMethodStarted"/> and
        /// <see cref="MockParser.ParseMethodEnded"/> events.
        /// </summary>
        /// <param name="method">
        /// The parsed element.
        /// </param>
        public void ParseMethod(MethodInfo method)
        {
            this.StartParseMethod(method);
            this.EndParseMethod(method);
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParsedEvent"/> event.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed element.
        /// </param>
        public void ParseEvent(EventInfo eventInfo)
        {
            if (eventInfo == null)
            {
                throw new ArgumentNullException("eventInfo");
            }

            if (this.ParsedEvent != null)
            {
                this.ParsedEvent(eventInfo);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParsedField"/> event.
        /// </summary>
        /// <param name="field">
        /// The parsed element.
        /// </param>
        public void ParseField(FieldInfo field)
        {
            if (field == null)
            {
                throw new ArgumentNullException("field");
            }

            if (this.ParsedField != null)
            {
                this.ParsedField(field);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParsedProperty"/> event.
        /// </summary>
        /// <param name="property">
        /// The parsed element.
        /// </param>
        public void ParseProperty(PropertyInfo property)
        {
            if (property == null)
            {
                throw new ArgumentNullException("property");
            }

            if (this.ParsedProperty != null)
            {
                this.ParsedProperty(property);
            }
        }

        /// <summary>
        /// Fires the <see cref="MockParser.ParsedParameter"/> event.
        /// </summary>
        /// <param name="parameter">
        /// The parsed element.
        /// </param>
        public void ParseParameter(ParameterInfo parameter)
        {
            if (parameter == null)
            {
                throw new ArgumentNullException("parameter");
            }

            if (this.ParsedParameter != null)
            {
                this.ParsedParameter(parameter);
            }
        }
    }
}
