﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the <see cref="Parser"/> class.
    /// </summary>
    [TestFixture]
    public class ParserTest
    {
        /// <summary>
        /// The parsed assemblies.
        /// </summary>
        private ICollection<string> startAssemblies;

        /// <summary>
        /// The parsed assemblies.
        /// </summary>
        private ICollection<string> endAssemblies;

        /// <summary>
        /// The parsed assembly attributes.
        /// </summary>
        private ICollection<string> attributes;

        /// <summary>
        /// The types parsed with <c>OnStartParseType</c>.
        /// </summary>
        private ICollection<string> startTypes;

        /// <summary>
        /// The types parsed with <c>OnEndParseType</c>.
        /// </summary>
        private ICollection<string> endTypes;

        /// <summary>
        /// The parsed fields.
        /// </summary>
        private ICollection<string> fields;

        /// <summary>
        /// The parsed constructors.
        /// </summary>
        private ICollection<string> startConstructors;

        /// <summary>
        /// The parsed constructors.
        /// </summary>
        private ICollection<string> endConstructors;

        /// <summary>
        /// The parsed events.
        /// </summary>
        private ICollection<string> events;

        /// <summary>
        /// The parsed properties.
        /// </summary>
        private ICollection<string> properties;

        /// <summary>
        /// The parsed methods.
        /// </summary>
        private ICollection<string> startMethods;

        /// <summary>
        /// The parsed methods.
        /// </summary>
        private ICollection<string> endMethods;

        /// <summary>
        /// Initializes the collections of the test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            this.startAssemblies = new LinkedList<string>();
            this.endAssemblies = new LinkedList<string>();
            this.attributes = new LinkedList<string>();
            this.startTypes = new LinkedList<string>();
            this.endTypes = new LinkedList<string>();
            this.fields = new LinkedList<string>();
            this.startConstructors = new LinkedList<string>();
            this.endConstructors = new LinkedList<string>();
            this.events = new LinkedList<string>();
            this.properties = new LinkedList<string>();
            this.startMethods = new LinkedList<string>();
            this.endMethods = new LinkedList<string>();
        }

        /// <summary>
        /// Tests the <see cref="Parser.Parse"/> method and the associated events.
        /// </summary>
        [Test]
        public void Parse()
        {
            IParser instance = new Parser();
            instance.ParseAssemblyStarted += this.OnParseAssemblyStarted;
            instance.ParseAssemblyEnded += this.OnParseAssemblyEnded;
            instance.ParsedAssemblyAttribute += this.OnParsedAssemblyAttribute;
            instance.ParseTypeStarted += this.OnParseTypeStarted;
            instance.ParseTypeEnded += this.OnParseTypeEnded;
            instance.ParsedField += this.OnParsedField;
            instance.ParseConstructorStarted += this.OnParseConstructorStarted;
            instance.ParseConstructorEnded += this.OnParseConstructorEnded;
            instance.ParsedEvent += this.OnParsedEvent;
            instance.ParsedProperty += this.OnParsedProperty;
            instance.ParseMethodStarted += this.OnParseMethodStarted;
            instance.ParseMethodEnded += this.OnParseMethodEnded;

            instance.Parse(MockAssembly.Instance);

            // Test the assembly parsing
            CollectionAssert.AreEquivalent(new string[] { "CodeReview.MockAssembly, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null" }, this.startAssemblies);
            CollectionAssert.AreEqual(this.startAssemblies, this.endAssemblies);

            // Test the assembly attributes parsing
            Assert.AreEqual(15, this.attributes.Count);

            // Test the types parsing
            Assert.AreEqual(6, this.startTypes.Count);
            CollectionAssert.AreEquivalent(
                new string[]
                {
                    "CodeReview.MockAssembly.MockPublicClass",
                    "CodeReview.MockAssembly.MockInternalClass",
                    "CodeReview.MockAssembly.MockInternalClass+PublicNestedClass",
                    "CodeReview.MockAssembly.MockInternalClass+InternalNestedClass",
                    "CodeReview.MockAssembly.MockInternalClass+ProtectedNestedClass",
                    "CodeReview.MockAssembly.MockInternalClass+PrivateNestedClass"
                },
                this.startTypes);
            CollectionAssert.AreEqual(this.startTypes, this.endTypes);

            // Test the fields parsing
            CollectionAssert.AreEquivalent(new string[] { "publicField", "protectedField", "internalField", "privateField" }, this.fields);

            // Test the constructors parsing
            Assert.AreEqual(9, this.startConstructors.Count);
            CollectionAssert.AreEqual(this.startConstructors, this.endConstructors);

            // Test the events parsing
            Assert.AreEqual(4, this.events.Count);
            CollectionAssert.AreEquivalent(new string[] { "PublicEvent", "ProtectedEvent", "InternalEvent", "PrivateEvent" }, this.events);

            // Test the properties parsing
            Assert.AreEqual(4, this.properties.Count);
            CollectionAssert.AreEquivalent(new string[] { "PublicProperty", "ProtectedProperty", "InternalProperty", "PrivateProperty" }, this.properties);

            // Test the methods parsing
            Assert.AreEqual(5, this.startMethods.Count);
            CollectionAssert.AreEquivalent(new string[] { "Finalize", "PublicMethod", "ProtectedMethod", "InternalMethod", "PrivateMethod" }, this.startMethods);
            CollectionAssert.AreEqual(this.startMethods, this.endMethods);
        }

        /// <summary>
        /// Called before parsing an assembly.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyStarted(Assembly assembly)
        {
            this.startAssemblies.Add(assembly.FullName);
        }

        /// <summary>
        /// Called after parsing an assembly.
        /// </summary>
        /// <param name="assembly">
        /// The parsed assembly.
        /// </param>
        private void OnParseAssemblyEnded(Assembly assembly)
        {
            this.endAssemblies.Add(assembly.FullName);
        }

        /// <summary>
        /// Called when parsing an attribute.
        /// </summary>
        /// <param name="attribute">
        /// The parsed attribute.
        /// </param>
        private void OnParsedAssemblyAttribute(CustomAttributeData attribute)
        {
            this.attributes.Add(attribute.ToString());
        }

        /// <summary>
        /// Called before parsing a type.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            this.startTypes.Add(type.FullName);
        }

        /// <summary>
        /// Called after parsing a type.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeEnded(Type type)
        {
            this.endTypes.Add(type.FullName);
        }

        /// <summary>
        /// Called when parsing a field.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            this.fields.Add(field.Name);
        }

        /// <summary>
        /// Called when parsing a constructor.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructorStarted(ConstructorInfo constructor)
        {
            this.startConstructors.Add(constructor.Name);
        }

        /// <summary>
        /// Called when parsing a constructor.
        /// </summary>
        /// <param name="constructor">
        /// The parsed constructor.
        /// </param>
        private void OnParseConstructorEnded(ConstructorInfo constructor)
        {
            this.endConstructors.Add(constructor.Name);
        }

        /// <summary>
        /// Called when parsing an event.
        /// </summary>
        /// <param name="eventInfo">
        /// The parsed event.
        /// </param>
        private void OnParsedEvent(EventInfo eventInfo)
        {
            this.events.Add(eventInfo.Name);
        }

        /// <summary>
        /// Called when parsing a property.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            this.properties.Add(property.Name);
        }

        /// <summary>
        /// Called when parsing a method.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            this.startMethods.Add(method.Name);
        }

        /// <summary>
        /// Called when parsing a method.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodEnded(MethodInfo method)
        {
            this.endMethods.Add(method.Name);
        }
    }
}
