﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Groups the utility methods that extracts the meta data of a type.
    /// </summary>
    internal static class Utilities
    {
        /// <summary>
        /// Extracts a field from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="name">
        /// The name of the field.
        /// </param>
        /// <returns>
        /// The meta data of the field or <c>null</c>.
        /// </returns>
        public static FieldInfo GetField(Type type, string name)
        {
            return type.GetField(name, Parser.AllDeclaredBindingFlags);
        }

        /// <summary>
        /// Extracts a constructor from a type definition based on its parameters.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="types">
        /// The types of the constructor parameters - use <see cref="Type.EmptyTypes"/>
        /// to extract the meta data of the default constructor.
        /// </param>
        /// <returns>
        /// The meta data of the property or <c>null</c>.
        /// </returns>
        public static ConstructorInfo GetConstructor(Type type, Type[] types)
        {
            return type.GetConstructor(Parser.AllDeclaredBindingFlags, null, types, null);
        }

        /// <summary>
        /// Extracts a property from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="name">
        /// The name of the property.
        /// </param>
        /// <returns>
        /// The meta data of the property or <c>null</c>.
        /// </returns>
        public static PropertyInfo GetProperty(Type type, string name)
        {
            return type.GetProperty(name, Parser.AllDeclaredBindingFlags);
        }

        /// <summary>
        /// Extracts an indexed property from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="indexerTypes">
        /// The types of the indexers.
        /// </param>
        /// <returns>
        /// The meta data of the indexed property or <c>null</c>.
        /// </returns>
        public static PropertyInfo GetIndexer(Type type, params Type[] indexerTypes)
        {
            return type.GetProperty("Item", Parser.AllDeclaredBindingFlags, null, null, indexerTypes, null);
        }

        /// <summary>
        /// Extracts a method from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="name">
        /// The name of the method.
        /// </param>
        /// <returns>
        /// The meta data of the method or <c>null</c>.
        /// </returns>
        public static MethodInfo GetMethod(Type type, string name)
        {
            return type.GetMethod(name, Parser.AllDeclaredBindingFlags);
        }

        /// <summary>
        /// Extracts a method from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="name">
        /// The name of the method.
        /// </param>
        /// <param name="types">
        /// The types of the parameters.
        /// </param>
        /// <returns>
        /// The meta data of the method or <c>null</c>.
        /// </returns>
        public static MethodInfo GetMethod(Type type, string name, Type[] types)
        {
            return type.GetMethod(name, Parser.AllDeclaredBindingFlags, null, types, null);
        }

        /// <summary>
        /// Extracts a parameter from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="methodName">
        /// The name of the method.
        /// </param>
        /// <param name="parameterName">
        /// The name of the parameter.
        /// </param>
        /// <returns>
        /// The meta data of the parameter or <c>null</c>.
        /// </returns>
        public static ParameterInfo GetParameter(Type type, string methodName, string parameterName)
        {
            MethodInfo method = GetMethod(type, methodName);
            if (method == null)
            {
                return null;
            }

            if (string.IsNullOrEmpty(parameterName))
            {
                return method.ReturnParameter;
            }
            else
            {
                return method.GetParameters().FirstOrDefault(p => p.Name == parameterName);
            }
        }

        /// <summary>
        /// Extracts an event from a type definition.
        /// </summary>
        /// <param name="type">
        /// The reference type.
        /// </param>
        /// <param name="eventName">
        /// The name of the event.
        /// </param>
        /// <returns>
        /// The meta data of the event or <c>null</c>.
        /// </returns>
        public static EventInfo GetEvent(Type type, string eventName)
        {
            return type.GetEvent(eventName, Parser.AllDeclaredBindingFlags);
        }
    }
}
