﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;

namespace CodeReview.Command
{
    /// <summary>
    /// Tests the <see cref="CommandLineParser"/> class.
    /// </summary>
    [TestFixture]
    public static class CommandLineParserTest
    {
        /// <summary>
        /// Tests the initial values of the default instance.
        /// </summary>
        [Test]
        public static void Constructor()
        {
            CommandLineParser instance = new CommandLineParser();

            Assert.AreEqual(CommandLineParser.DefaultArgumentAffectation, instance.ArgumentAffectation);
            Assert.AreEqual(CommandLineParser.DefaultArgumentPrefix, instance.ArgumentPrefix);
            Assert.AreEqual(CommandLineParser.DefaultValueSeparator, instance.ValueSeparator);
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with a simple argument.
        /// </summary>
        [Test]
        public static void TokenizeWithoutValue()
        {
            string[] args = new string[] { "/a" };

            CommandLineParser instance = new CommandLineParser();
            IDictionary<Option, OptionValues> actual = instance.Tokenize(args);

            Assert.IsNotNull(actual);
            Assert.AreEqual(1, actual.Count);
            Assert.AreEqual(Option.Assembly, actual.First().Key);
            Assert.AreEqual(Option.Assembly, actual.First().Value.Option);
            CollectionAssert.IsEmpty(actual.First().Value.Values);
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with a key/value argument.
        /// </summary>
        [Test]
        public static void TokenizeWithValue()
        {
            string[] args = new string[] { "/a=value" };

            CommandLineParser instance = new CommandLineParser();
            IDictionary<Option, OptionValues> actual = instance.Tokenize(args);

            Assert.IsNotNull(actual);
            Assert.AreEqual(1, actual.Count);
            Assert.AreEqual(Option.Assembly, actual.First().Key);
            Assert.AreEqual(Option.Assembly, actual.First().Value.Option);

            ICollection<string> values = actual.First().Value.Values;
            CollectionAssert.IsNotEmpty(values);
            Assert.AreEqual(1, values.Count);
            Assert.AreEqual("value", values.First());
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with a key/values argument.
        /// </summary>
        [Test]
        public static void TokenizeWithValues()
        {
            string[] args = new string[] { "/a=value1,value2" };

            CommandLineParser instance = new CommandLineParser();
            IDictionary<Option, OptionValues> actual = instance.Tokenize(args);

            Assert.IsNotNull(actual);
            Assert.AreEqual(1, actual.Count);
            Assert.AreEqual(Option.Assembly, actual.First().Key);
            Assert.AreEqual(Option.Assembly, actual.First().Value.Option);

            ICollection<string> values = actual.First().Value.Values;
            CollectionAssert.IsNotEmpty(values);
            Assert.AreEqual(2, values.Count);
            Assert.AreEqual("value1", values.First());
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with an invalid argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(CommandException))]
        public static void TokenizeInvalidArgument()
        {
            string[] args = new string[] { "/a=" };

            CommandLineParser instance = new CommandLineParser();
            instance.Tokenize(args);
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with an unknown option.
        /// </summary>
        [Test]
        [ExpectedException(typeof(CommandException))]
        public static void TokenizeUnknownError()
        {
            string[] args = new string[] { "/s=s" };

            CommandLineParser instance = new CommandLineParser();
            instance.Tokenize(args);
        }

        /// <summary>
        /// Tests the <see cref="CommandLineParser.Tokenize"/> method
        /// with no argument.
        /// </summary>
        [Test]
        public static void TokenizeEmpty()
        {
            CommandLineParser instance = new CommandLineParser();
            IDictionary<Option, OptionValues> actual = instance.Tokenize(new string[0]);

            Assert.IsNotNull(actual);
            Assert.AreEqual(0, actual.Count);
        }
    }
}
